import pandas as pd
import numpy as np
import datetime
import matplotlib.pyplot as plt
from datetime import timedelta

#Site to get data http://dhime.ideam.gov.co/atencionciudadano/

class Ideam():
    def __init__(self, path, encoding='utf-8', sep='|', type='t_maxmin', serie='diário'):
        def type_data(df, type):
            return type

        def dataframe():
            if type == 't_maxmin':
                df = pd.read_csv(path, encoding=encoding,
                                 index_col=False, sep=sep)
                df["Fecha"] = pd.to_datetime(df['Fecha']).dt.date
                return df

        import pandas as pd
        self.dataframe = dataframe()
        self.type = "original" 
        self.len = len(self.dataframe)
        self.type_data = type_data(self.dataframe, type)
        self.startdate = self.dataframe['Fecha'][0]
        self.enddate = self.dataframe['Fecha'][len(self.dataframe) - 1]
    def byYear(self, year, var):
        """
        This function get the file and the var and return the data from this year

        :param year: type int
        :param var: type str with data var to get
        :return: data from colum 'var' and data in year 'year'
        """

        inicio, final = 0, 0
        for linha in range(self.len):
            l = linha
            if self.date(linha).year == year:
                inicio = linha  # linha de inicio
                while self.date(l).year == year and l < self.len - 1:
                    l += 1
                final = l
                break
        return np.array(self.dataframe[f'{var}'][inicio:final])

    def report(self, path):
        """
        Save a txt file with importants informations

        """
        self.format1()
        if (self.type_data == "precipitation"):
            with open(path, 'w') as f:
                #f.write(f"Station {self.city} {self.code}\n")
                f.write(f"Empty data of : {self.empty_data()}\n")
                f.write(f"Max value: {np.max(self.dataframe.iloc[:, 1])}\n")
                v= self.dataframe.iloc[np.argmax(self.dataframe.iloc[:, 1]),0]
                f.write(f"Date of max value: {v.day}/{v.month}/{v.year}\n")
                f.write(f"Min value: {np.min(self.dataframe.iloc[:, 1])}\n")
                v= self.dataframe.iloc[np.argmin(self.dataframe.iloc[:, 1]),0]
                f.write(f"Date of min value: {v.day}/{v.month}/{v.year}\n")
                f.close()
    def byMonth(self, year, month, var):
        """
        English:
            With array dayly with many years, we select only year and month request
        Português:
            Dado um array diário com vários anos, ele seleciona e retorna apenas uma região daquele mês pedido
        :param year:
        :param month:
        :param var:
        :return: return a array with data in date requests
        """

        inicio, final = 0, 0
        for linha in range(self.len):
            l = linha
            if self.date(linha).year == year and self.date(linha).month == month:
                inicio = linha  # start line
                while self.date(l).year == year and self.date(l).month == month and l < self.len - 1:
                    l += 1
                final = l
                break
        return np.array(self.dataframe[f'{var}'][inicio:final])

    def rx5day(self, with_x=False):
       import indice
       a = indice.Indice(self.dataframe)
       a = a.rx5day(with_x=with_x)
       return a

    def rx5day_month(self, with_x=False):
        k = 0
        m = x = y = np.array([0])
        kano = 0
        for ano in range(self.startdate.year, self.enddate.year + 1):
            for mes in range(1, 13):
                while self.date(k).month == mes:  # while in the month
                    t = 0
                    for k2 in range(5):
                        if k == self.len - 1:
                            break
                        t += float(self.dataframe["Valor"][k])
                        k += 1
                    if t > y[kano]:
                        y[kano] = t
                        m[kano] = int(mes)
                    if k == self.len - 1:
                        break
            y = np.append(y, 0)
            m = np.append(m, 0)
            kano += 1

        return np.arange(self.startdate.year, self.enddate.year + 1), y[:-1], m[:-1]

    def tx(self, with_x=False):
        """
        English:
            Find the absolute max temperature from maximum temperature dairy, by year
            The data needs be daily

        Português:
            Encontra a temperatura máxima da máxima anual.
            Os dados necessitam ser diários
        :return: return
        """
        if self.type_data != 't_maxmin':
            raise "The file has not 'type'= 'temp_maxmin' "

        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        return ind.tx(with_x=with_x)

    def tn(self, with_x=False):
        '''
        English:
            Find the absolute max temperature from minimun temperature dairy, by year
            The data needs be diary

        Portuguese:
                Encontra a temperatura máxima da mínima anual.
                Os dados necessitam ser diários
        :return: Return array with data or data and date
        '''
        if self.type_data != 't_maxmin':
            raise "The file is not cited 'type_data' as temp_maxmin "

        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.tn(with_x=with_x)
        return ind

    def tx90p(self, with_x=False):
        '''
        English:
            Find the 90% percentil of tx in all array

        Português:
            Encontra o percentil de 90% do TX para o mesmo período

        :param with_x: True or False, return date array from data
        :return: Return array with data or data and date
        '''
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        return ind.tx90p(with_x=with_x)

    def tn90p(self, with_x=True):
        '''
        English:
            Find the 90% percentil of tn in all array

        Português:
            Encontra o percentil de 90% do tn para o mesmo período

        :param with_x: True or False, se deseja que venha o valor da data também
        :return: Return array with data or data and date
        '''

        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        return ind.tn90p(with_x=with_x)

    def prcptot(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataset)
        ind = ind.prcptot(with_x=with_x)
        return ind

    def prcptot_monthly(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.prcptot_monthly(with_x=with_x)
        return ind

    def date(self, line_number):
        if self.type == 'format1':
            return self.dataframe['time'][line_number]
        return self.dataframe['Fecha'][line_number]

    def empty_data(self, type="absolute"):
        if self.type != "format1":
            self.format1()
        if (self.type_data == "precipitation"):
            if type == 'relative':
                s = np.sum(np.isnan(self.dataframe.iloc[:,1]))
                s = (s / self.len) * 100
                return s
            elif (type == "absolute"):
                return np.sum(np.isnan(self.dataframe.iloc[:,1])) # The absolute

    def format1(self, comma_to_dot=True, grow=True, years=(0,0)):
        """
                We change the file format to
                1) put date that can be jumped
                2) organize the file from to smaller to larger based in time
                3) Change the ',' to '.'
                4) Remove same values from file
                5) Change the name of time variable to time
                6) padronize a file to work with Ana, Inemet, Merge and etc
                7) have option to select a series of year start and year end. The date that not exist, are created and set NaN


                The out file is in format

                                time    pr
                                1997-07-01  0.1
                                1997-07-02  0.3
                                ....
                                ...
                                ...
                                2020-01-01  0.3
                                2020-01-02  0.0

                """
        import pandas as pd
        if years != (0,0) and self.type != "format1": # if is not format1 and select date
            self.dataframe['Fecha'] = pd.to_datetime(self.dataframe['Fecha'])
            self.dataframe['Valor'] = np.array(self.dataframe['Valor']).astype(float)
            self.dataframe = self.dataframe.rename(columns={'Fecha': 'time', 'Valor': 'pr'})
            if comma_to_dot:
                self.dataframe = self.dataframe.replace({',': '.'}, regex=True)

            data = self.dataframe["time"]
            pr = self.dataframe["pr"]

            # chech if have date jump
            a = pd.date_range(start=str(years[0]), end=str(years[1]), freq='D')
            a=a[:-1]
            pr_def = np.full(len(a), np.nan)
            for i in range(len(data)):
                ind = np.argwhere(data[i] == a)
                pr_def[ind] = pr[i]

            df = pd.DataFrame(list(zip(a, pr_def)), columns=['time', 'pr'])

            # atualize the format and len
            self.type = "format1"
            df = df[(df.iloc[:, 0].dt.year <= years[1])]
            self.dataframe = df[df.iloc[:, 0].dt.year >= years[0]]
            self.len = len(self.dataframe)
            self.enddate = df.iloc[-1,0] 
            self.startdate = df.iloc[0, 0]

            # atualize the format and len
            self.type = "format1"
            self.len = len(self.dataframe)

        elif self.type != "format1":
            self.dataframe['Fecha'] = pd.to_datetime(self.dataframe['Fecha'])
            self.dataframe['Valor'] = np.array(self.dataframe['Valor']).astype(float)
            self.dataframe = self.dataframe.rename(columns={'Fecha': 'time', 'Valor': 'pr'})
            if comma_to_dot:
                self.dataframe = self.dataframe.replace({',': '.'}, regex=True)

            colum = self.dataframe.columns
            data = pd.to_datetime(self.dataframe[colum[0]])
            pr = self.dataframe[colum[1]].to_numpy()

            # chech if have date jump
            a = pd.date_range(start=self.startdate, end=self.enddate, freq='D')
            a=a[:-1]
            pr_def = np.full(len(a), np.nan)
            for i in range(len(data)):
                ind = np.argwhere(data[i] == a)
                pr_def[ind] = pr[i]

            self.dataframe = pd.DataFrame(list(zip(a, pr_def)), columns=['time', 'pr'])


        self.type_data = 'precipitation'
        self.type = "format1"

    def get_year(self,year, with_x=False, return_x0=False):
        """
        English:
            From array multi-year, return a year specific

        Português:
            Do array, retorna um ano específico

        :param year: can by a Int, or list of Int
        :param with_x: True or False, to return date array or no
        :param return_x0: Return the index init
        :return:
        """
        self.format1()
        if isinstance(year, int): # if is only a year
            try:
                a = pd.to_datetime(
                    self.dataframe['time']).dt.year.to_numpy() == year  # array with true or false if the year is that we want
            except:
                raise f"The Data don't have the year {year}"

            a= np.where(a== True)[0] # Return array with index of True and correct the list format
            if with_x:
                return self.dataframe['time'][a], self.dataframe['pr'][a]
            elif return_x0:
                return a[0], self.dataframe['time'][a]
            return self.dataframe['pr'][a]
        else: # is many years
            xf = pd.Series([])
            yf = np.array([])
            for i in year:
                try:
                    a = pd.to_datetime(
                        self.dataframe[
                            'time']).dt.year.to_numpy() == i # array with true or false if the year is that we want
                except:
                    raise f"The Data don't have the year {i}"

                a = np.where(a == True)[0]  # Return array with index of True and correct the list format
                xf = pd.concat([xf, self.dataframe['time'][a]])
                yf = np.concatenate([yf, self.dataframe['pr'][a].to_numpy()])
            if with_x:
                return xf, yf
            elif return_x0:
                raise "This function exist only to one year"
            return yf
    def get_month(self, month, year, with_x=False):
        """
        English:
            From array muti-year, return a month specific

        Portuguese:
            Do array, retorna um mês específico
        :param month: The month to select as Int, or list of month to get
        :param year: The year to select
        :with_x: return the data, True or Not
        :return: Array with data of month select
        """
        if self.type_data != "format'":
            self.format1()
        #x0 is the index wen start the year that we want
        x0, time = self.get_year(year, return_x0=True)
        if isinstance(month, int):
            try:
                a = time.dt.month.to_numpy() == month  # array with true or false if the year is that we want
            except:
                raise f"The data has't the month {month}"
            a = np.where(a == True)[0]  # Return array with index of True and correct the list format
            a += x0
            if with_x:
                return self.dataframe['time'][a], self.dataset['pr'][a]
            return self.dataframe['pr'][a]
        else:
            xf = pd.Series([])
            yf = np.array([])
            for i in month:
                try:
                    a = time.dt.month.to_numpy() == i  # array with true or false if the year is that we want
                except:
                    raise f"The data has't the month {i}"
                a = np.where(a == True)[0]  # Return array with index of True and correct the list format
                a += x0
                xf = pd.concat([xf, self.dataframe['time'][a]])
                yf = np.concatenate([yf, self.dataframe['pr'][a].to_numpy()])
            if with_x:
                return xf, yf
            return yf
    def to_csv(self, path):
        """
         English:
            This function save the file that is save in .dataframe
        Português:
            Essa função salva o arquivo salvo dentro do .dataframe como um arquivo csv

        :param path:  str path to safe file and format .csv
        :return:
        """
        self.dataframe.to_csv(path)
    def to_netcdf(self,path):
        import xarray as xr
        t = self.dataframe.to_xarray()
        t = xr.Dataset(t)
        t.to_netcdf(path)



import datetime

import numpy as np
import pandas as pd

# The index are based in: https://www.climdex.org/learn/indices/
class Indice:
    def __init__(self, df):
        self.dataframe = df
        self.len = len(self.dataframe)

    def date(self, linha):
        '''
                Encontra a altura da estação no dataframe
        '''
        import datetime
        df = self.dataframe
        df = df.iloc[linha, 0]
        try:  # Caso já esteja em Datetime
            temp = df.year
            del temp
            return df
        except:
            df = datetime.datetime.strptime(df, '%Y-%m-%d')
            return df

    def ByYear(self, year, var):
        """
        :param year: A data que queremos
        :param var: A variável do dataframe
        :return: Retorna um array com todos os elementos do ano que você digitou
        """
        # percorre procurando por ano
        inicio, final = 0, 0
        for linha in range(self.len):
            n = 0
            l = linha
            if self.date(linha).year == year:
                inicio = linha  # linha de inicio
                while l <= self.len - 1 and self.date(l).year == year:
                    l += 1
                final = l
                break
        return np.array(self.dataframe[f'{var}'][inicio:final])

    def ByMonth(self, year, month, var):
        """
        Dado um array diário com vários anos, ele seleciona e retorna apenas uma região daquele mês pedido
        :param year:
        :param month:
        :param var:
        :return:
        """
        # percorre procurando por ano
        inicio, final = 0, 0
        for linha in range(self.len):
            n = 0
            l = linha
            if self.date(linha).year == year and self.date(linha).month == month:
                inicio = linha  # linha de inicio
                while self.date(l).year == year and self.date(l).month == month and l < self.len - 1:
                    l += 1
                final = l
                break
        return np.array(self.dataframe[f'{var}'][inicio:final])
        #  íncides de temperatura

    def tx(self, with_x=False, type_data='t_maxmin', var='t_maxmin'):
        """
        Encontra a temperatura máxima da máxima anual.
        Os dados necessitam ser diários
        :param with_x:
        :param type_data:
        :param var:
        :return:
        """
        if var.lower() not in ['t_maxmin', 'tmax', 'max']:
            raise 'O arquivo não está citado "type" como temp_maxmin'
        if type_data == 'pr':
            raise "The file is precipitation data"
        tx = np.array([])  # variável
        inicio = self.date(0).year
        fim = self.date(self.len - 1).year

        for k in range(inicio, fim+1):  # percorre os anos
            tx = np.append(tx, np.nanmax(self.ByYear(k, "max")))
        if with_x:
            return np.arange(inicio, fim + 1), tx
        return tx

    def tn(self, with_x=False, type_data='t_maxmin', var='min'):
        """
        Encontra a temperatura máxima da mínima anual.
            Os dados necessitam ser diários
        :param with_x:
        :param type_data:
        :param var:
        :return:
        """
        if type_data != 't_maxmin':
            raise 'O arquivo não está citado "type" como temp_maxmin'

        if type_data == 'pr':
            raise
        tn = np.array([])  # variável
        inicio = self.date(0).year
        fim = self.date(self.len-1).year
        for k in range(inicio, fim + 1):  # percorre os anos
            tn = np.append(tn, np.nanmax(self.ByYear(k, "min")))

        if with_x:
            return np.arange(inicio, fim + 1), tn
        return tn

    def tx90p(self, with_x=False, type_data='t_maxmin', var='tmax'):
        '''
        Encontra o percentil de 90% do TX para o mesmo período

        :param with_x:
        :return:
        '''

        x = np.arange(self.date(0).year, self.date(self.len - 1).year)
        cont = i = 0
        tx90p = [0]  # O vetor começa em 0
        for ano in x:
            dias = 0
            ptx = np.nanpercentile(self.ByYear(ano, var=f'{var}'), 90)
            while self.date(cont).year == ano:  # enquanto trabalharmos em um mesmo ano
                if self.dataframe[f'{var}'][cont] > ptx:  # compara com o percentil 90% do TX anual
                    tx90p[i] += 1
                cont += 1
                dias += 1
            tx90p[i] = 100 * tx90p[i] / dias  # Deixa em percentual anual
            i += 1
            tx90p.append(0)

        if with_x:
            return x, tx90p[0:-1]
        return tx90p[0:-1]

    def tn90p(self, with_x=True, var='min'):
        '''

        :param with_x: True or False, se deseja que venha o valor da data também
        :var:
        :return:
        '''
        x = np.arange(self.date(0).year, self.date(self.len - 1).year)
        cont = i = 0
        tn90p = [0]  # O vetor começa em 0
        for ano in x:
            dias = 0
            ptx = np.nanpercentile(self.ByYear(ano, var=f'{var}'), 90)
            while self.date(cont).year == ano:  # enquanto trabalharmos em um mesmo ano
                if self.dataframe[f'{var}'][cont] > ptx:  # compara com o percentil 90% do TX anual
                    tn90p[i] += 1
                cont += 1
                dias += 1
            tn90p[i] = 100 * tn90p[i] / dias  # Deixa em percentual anual
            i += 1
            tn90p.append(0)

        if with_x:
            return x, tn90p[0:-1]
        return tn90p[0:-1]

    def tn10p(self, with_x=False, type_data='t_maxmin'):
        '''

            :param with_x: True or False, se deseja que venha o valor da data também
            :return:
            '''
        if type_data == 't_maxmin':
            x = np.arange(self.date(0).year, self.date(self.len - 1).year)
            cont = i = 0
            tn90p = [0]  # O vetor começa em 0
            for ano in x:
                dias = 0
                ptx = np.nanpercentile(self.ByYear(ano, var='min'), 10)
                while self.date(cont).year == ano:  # enquanto trabalharmos em um mesmo ano
                    if self.dataframe['min'][cont] > ptx:  # compara com o percentil 90% do TX anual
                        tn90p[i] += 1
                    cont += 1
                    dias += 1
                tn90p[i] = 100 * tn90p[i] / dias  # Deixa em percentual anual
                i += 1
                tn90p.append(0)

            if with_x:
                return x, tn90p[0:-1]
            return tn90p[0:-1]

    def fd(self, with_x=False, type_data='t_maxmin', var='time'):
        """
        Number of frost days by year
        :param type_data:  Tipo de informação contida no array
        :param var: Nome da variável
        :return: Retorna A quantidade de dias com Temperatura <0°C por ano
        """
        if type_data == 't_maxmin':
            fd = np.array([])  # Armazena os valores
            inicio = self.date(0).year  # ano de inicio
            fim = self.date(self.len).year  # Ano de termino
            ind = 0
            for ano in range(inicio, fim + 1):
                t = self.ByYear(ano, var)  # vetor com dados de um ano
                fd = np.append(fd, np.sum(t < 0))
            if with_x:
                return pd.to_datetime(np.arange(inicio, fim)), fd
            return fd

    def su(self, with_x=False, type_data='t_maxmin', var='time'):
        """
        Number of summer days by year
        :param with_x: Retorna o vetor com datas ou não (True, False)
        :param type_data: Tipe de data que está entrando
        :param var: Como a variável está denominada no arquivo
        :return: Retorna a quantidade de dias da temperatura máxima > 25°C por ano
        """
        if type_data == 't_maxmin':
            su = np.array([])  # Vetor com dados
            inicio = self.date(0).year  # Ano de inicio
            fim = self.date(self.len).year  # Ano de termino
            for ano in range(inicio, fim + 1):
                t = self.ByYear(ano, var)  # vetor com dados de um ano
                su = np.append(su, np.sum(t > 25))  # adiciona a soma do vetor True onde t>25
            if with_x:
                return pd.to_datetime(np.arange(inicio, fim)), su
            return su

    def etr(self, with_x=False, type_data='t_maxmin', var=('tmax', 'tmin')):
        """
        Extreme temperature range of monthly
        :param with_x: Retorna o vetor com datas ou não (True, False)
        :param type_data: Tipe de data que está entrando
        :param var: um vetor em que a primeira componente é o nome da temperatura máxima e o segundo da temperatura mínima
        :return: Retorna A temperatura máxima mensal menos a mínima
        """
        etr = np.array([])  # variável de saída
        inicio = self.date(0).year  # ano de inicio
        fim = self.date(self.len).year  # Ano de termino
        for ano in range(inicio, fim + 1):
            for mes in range(1, 13):
                etr = np.append(etr, np.max(self.ByMonth(ano, mes, var[0])) - np.max(self.ByMonth(ano, mes, var[1])))
        if with_x:
            return np.arange(np.datetime64(f"{inicio}-{self.date(0).month}-01"),
                             np.datetime64(f"{fim + 1}-{self.date(self.len).month}-01"),
                             np.timedelta64(1, 'M'),
                             dtype='datetime64[M]'), etr
        return etr

    # índices de chuvas
    def rnnmm(self, number, with_x=False, type_data='pr', var='pr'):
        """
        Annual count of days when PRCP ≥ nn mm, where nn is a user-defined threshold
        Let RRij be the daily precipitation amount on day i in period j. Count the number of days where RRij ≥ nnmm.
        :param with_x: Retorna o array com as datas de cada dado
        :param type_data:
        :param var:
        :return:
        """
        if type_data == 'pr':
            rnnmm = np.array([])  # Vetor com dados
            inicio = self.date(0).year  # Ano de inicio
            fim = self.date(self.len-1).year  # Ano de termino
            for ano in range(inicio, fim + 1):
                t = self.ByYear(ano, var)  # vetor com dados de um ano
                rnnmm = np.append(rnnmm, np.sum(t > number))  # adiciona a soma do vetor True onde t>25
            if with_x:
                print(pd.to_datetime(np.arange(inicio, fim+1)))
                return np.arange(inicio, fim+1), rnnmm
            return rnnmm

    def rx5day(self, valor=1, retornar='number_of_5day_heavy_precipitation_periods_per_time_period', time='y',
               selbyyear=0,
               with_x=False, type_data='pr'):
        '''
        Rx5day analisa o aculmulado em de 5 dias em 5 dias durante todo o ano e retorna a data com maior valor
        :param valor: valor fixo do mínimo que consideramos
        :param retornar:Opções para retornar: highest_five_day_precipitation_amount_per_time_period, que é uma lista 0 é o numero e 1 é o mês
        ; number_of_5day_heavy_precipitation_periods_per_time_period
        :time:
        :return:
        '''

        import datetime
        import pandas as pd
        if type_data == "pr":  # if data are precipitation
            df = self.dataframe
            df.iloc[:, 0] = pd.to_datetime(df.iloc[:, 0])
            # deixa apenas os dados de precipitação
            number_of_5day_heavy_precipitation_periods_per_time_period = 0
            date = [0]
            maior = {0: 0, 1: 0}

            if time == 'y':
                data = pd.to_datetime(df.iloc[:, 0])
                serie = df.iloc[:, 1]
                start = self.date(0).year
                resul = [0]
                cont = r = 0
                while cont < len(data) - 1:  # while is not the end year
                    temp = 0
                    for k in range(5):
                        if cont + k < len(data) - 2 and self.date(
                                cont + k).year == start:  # Se continuamos no mesmo ano
                            temp += float(serie[cont + k])

                        else:
                            break
                    if temp > resul[r]:  # Se a sequência atual é maior que a anterior
                        resul[r] = temp
                        date[r] = self.date(cont).month

                    if self.date(cont).year != start:  # pulamos de ano
                        r += 1
                        cont += 1
                        resul.append(0)
                        date.append(0)
                        start += 1
                    else:
                        cont += 5
                number_of_5day_heavy_precipitation_periods_per_time_period = resul

            # Retorna
            if retornar == 'highest_five_day_precipitation_amount_per_time_period':
                if with_x:  # retornamos o eixo x também
                    return np.arange(self.date(0).year, self.date(-1).year + 1), maior
                else:
                    return maior
            elif retornar == 'number_of_5day_heavy_precipitation_periods_per_time_period':
                return (range(self.date(0).year, self.date(-1).year + 1),
                        number_of_5day_heavy_precipitation_periods_per_time_period)
            elif retornar == "date":
                return  (range(self.date(0).year, self.date(-1).year + 1), date)

        raise "O dado não está definido como precipitação (pr)"  # Caso não, erro

    def cdd(self, with_x=False, type_data='pr'):
        serie = np.array(self.dataframe.iloc[:, 1]).astype(float)
        start = self.date(0).year
        resul = [0]
        cont = r = 0
        while cont <= len(self.dataframe) - 1:  # Enquanto não for o último ano
            temp = 0
            if cont < len(self.dataframe) - 1:
                if serie[cont] <= 1 and isinstance(serie[cont], float):  # Se não choveu
                    while serie[cont] <= 1 and self.date(cont).year == start and cont < len(
                            self.dataframe) - 1:  # Enquanto não chover, conta os dias
                        temp += 1
                        cont += 1
                else:
                    cont += 1

            if temp > resul[r]:  # Se a sequência atual é maior que a anterior
                resul[r] = temp
            if self.date(cont).year != start:  # pulamos de ano
                r += 1
                cont += 1
                resul.append(0)
                start += 1
            else:
                cont += 1
        if with_x:
            return np.arange(self.date(0).year, self.date(-1).year + 1), resul
        return resul

    def cwd(self, with_x=False, type_data='pr'):
        serie = np.array(self.dataframe.iloc[:, 1]).astype(float)
        start = self.date(0).year
        resul = np.array([0])
        cont = r = 0
        while cont <= len(self.dataframe) - 1:  # Enquanto não for o último ano
            temp = 0
            # print(cont)
            if serie[cont] >= 1 and isinstance(serie[cont], float):  # Se não choveu ou o dado é inválido
                while serie[cont] >= 1 and self.date(cont).year == start and cont < len(
                        self.dataframe) - 1:  # Enquanto não chover, conta os dias
                    temp += 1
                    cont += 1
            else:
                cont += 1

            if temp > resul[r]:  # Se a sequência atual é maior que a anterior

                resul[r] = temp

            if self.date(cont).year != start:  # pulamos de ano
                r += 1
                cont += 1
                resul = np.append(resul, 0)
                start += 1
            else:
                cont += 1
        if with_x:
            return np.arange(self.date(0).year, self.date(-1).year + 1), resul
        return resul

    def prcptot(self, with_x=False, type_data='pr', var='pr'):
        if type_data == 'pr':
            start = self.date(0).year
            r = np.array([])
            for ano in range(start, self.date(self.len-1).year+1):
                r = np.append(r, np.nansum(self.ByYear(ano,var)))
            if with_x:
                return np.arange(self.date(0).year, self.date(-1).year), r
            return r
        else:
            raise "This index can only be calculated with precipitation (type_data='pr')"

    def prcptot_monthly(self, with_x=False, type_data='pr', var='pr'):
        if type_data == 'pr':
            prcptot = np.array([])
            inicio = self.date(0).year  # Ano de inicio
            fim = self.date(self.len-1).year  # Ano de termino
            for ano in range(inicio, fim + 1):
                for mes in range(1, 13):
                    if ano == inicio: # Se é o ano inicial
                        if mes>=self.date(0).month: # se o mês for válido
                            prcptot = np.append(prcptot, np.nansum(self.ByMonth(ano, mes, var=var)))
                    elif ano == fim: # se é o último ano
                        if mes <= self.date(self.len-1).month:
                            prcptot = np.append(prcptot, np.nansum(self.ByMonth(ano, mes, var=var)))
                    else:
                        prcptot = np.append(prcptot, np.nansum(self.ByMonth(ano, mes, var=var)))
            if with_x:
                return pd.date_range(start =f'{inicio}-{self.date(0).month}-01',end =f'{fim}-{self.date(self.len-1).month}-1', freq = '1M'), prcptot[:-1]
            return prcptot
        else:
            raise "This index can only be calculated with precipitation (type_data='pr')"

    def r99p(self, with_x=False, type_data='pr', var='pr'):
        #Refazer
        if type_data == 'pr':
            x = np.arange(self.date(0).year, self.date(self.len-1).year+1)
            cont = i = 0
            tn90p = [0]  # O vetor começa em 0
            for ano in x:
                dias = 0
                ptx = np.percentile(self.ByYear(ano, var=var), 99)
                while cont < self.len and self.date(cont).year == ano:  # enquanto trabalharmos em um mesmo ano
                    if self.dataframe[f'{var}'][cont] > ptx:  # compara com o percentil 99% do percentil anual
                        tn90p[i] += 1
                    cont += 1
                    dias += 1
                tn90p[i] = 100 * tn90p[i] / dias  # Deixa em percentual anual
                i += 1
                tn90p.append(0)

            if with_x:
                return x, tn90p[0:-1]
            return tn90p[0:-1]

    def r95p(self, with_x=False, type_data='pr', var='pr'):
        """
        :param with_x:
        :param type_data:
        :param var:
        :return: Retorna o vetor anual em que o percentil de 95
        """
        if type_data == 'pr':
            # Refazer, dar um jeito no nan
            x = np.arange(self.date(0).year, self.date(self.len - 1).year + 2)
            cont = i = 0
            r95p = [0]  # O vetor começa em 0
            for ano in x:
                dias = 0
                ptx = np.nanpercentile(self.ByYear(ano, var=var), 95)
                while self.date(
                        cont).year == ano:  # enquanto tra                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          balharmos em um mesmo ano
                    if self.dataframe['pr'][cont] > ptx:  # compara com o percentil 90% do TX anual
                        r95p[i] += 1
                    cont += 1
                    dias += 1
                r95p[i] = 100 * r95p[i] / dias  # Deixa em percentual anual
                i += 1
                r95p.append(0)

            if with_x:
                return x, r95p[0:-1]
            return r95p[0:-1]

    def r10mm(self, with_x=False, type_data='pr', var='pr'):
        if type_data == 'pr':
            r10mm = np.array([])  # Vetor com dados
            inicio = self.date(0).year  # Ano de inicio
            fim = self.date(self.len-1).year  # Ano de término
            for ano in range(inicio, fim + 1):
                t = self.ByYear(ano, var)  # vetor com dados de um ano
                r10mm = np.append(r10mm, np.nansum(t >= 10))  # adiciona a soma do vetor True onde t>=10
            if with_x:
                return np.arange(inicio, fim+1), r10mm
            return r10mm

    def r20mm(self, with_x=False, type_data='pr', var='pr'):
        """

        :param with_x:
        :param type_data:
        :param var:
        :return:
        """
        if type_data == 'pr':
            r10mm = np.array([])  # Vetor com dados
            inicio = self.date(0).year  # Ano de inicio
            fim = self.date(self.len-1).year  # Ano de termino
            for ano in range(inicio, fim + 1):
                t = self.ByYear(ano, var)  # vetor com dados de um ano
                r10mm = np.append(r10mm, np.sum(t >= 20))  # adiciona a soma do vetor True onde t>=20
            if with_x:
                return np.arange(inicio, fim+1), r10mm
            return r10mm

    def r99pTOT(self, with_x=False, type_data='pr', var='pr'):
        """
        Contribution to total precipitation from very wet days

        :param with_x:
        :param type_data:
        :param var
        :return:
        """
        if type_data == 'pr':
            inicio = self.date(0).year  # Ano de inicio
            fim = self.date(self.len-1).year  # Ano de termino
            r99ptot = 100 * (self.r99p()/self.prcptot())
            if with_x:
                return np.arange(inicio, fim+1), r99ptot
            return r99ptot

    def rx1day_anual(self, with_x=False, type_data='pr', var='pr'):
        """

        :param with_x:
        :param type_data:
        :param var:
        :return:
        """
        if type_data == 'pr':
            rx1day = np.array([])
            inicio = self.date(0).year  # Ano de inicio
            fim = self.date(self.len-1).year  # Ano de termino
            for ano in range(inicio, fim + 1):
                rx1day = np.append(rx1day, np.nanmax(self.ByYear(ano, var=var)))  # o máximo de precipitação diária no ano
            if with_x:
                return np.arange(inicio, fim+1), rx1day
            return rx1day

    def rx1day_monthly(self, with_x=False, type_data='pr', var='pr'):
        """

        :param with_x:
        :param type_data:
        :param var:
        :return: Retorna um array com a precipitação máxima em cada mês
        """

        if type_data == 'pr':
            rx1day = np.array([])
            inicio = self.date(0).year  # Ano de inicio
            fim = self.date(self.len-1).year  # Ano de termino
            for ano in range(inicio, fim + 1):
                for mes in range(1, 13):
                    try:
                        rx1day = np.append(rx1day,
                                           np.max(self.ByMonth(ano, mes, var=var)))  # o máximo de precipitação diária no ano
                    except:
                        pass
            if with_x:
                return pd.date_range(start =f'{inicio}-{self.date(0).month}-01',end =f'{fim}-{self.date(self.len-1).month}-1', freq = '1M'), rx1day[:-1]
            return rx1day
    def Mann_Kendall_Test(self, type_data = None):
        def sign(v):
            if (v>0):
                return 1
            elif (v==0):
                return 0
            else:
                return -1
        def p_value(s):
            if (s>0):
                return (s-1)/np.std(np.array(self.dataframe.iloc[:,1]))  
            elif (s==0):
                return 0
            else:
                return (s+1)/np.std(np.array(self.dataframe.iloc[:, 1]))
        if (type_data == "precipitation"):
            s = 0
            for i in range(self.len):
                for j in range(i, self.len):
                    s += sign(self.dataframe.iloc[j,1] - self.dataframe.iloc[i, 1])
            return (s, p_value(s)) 
        elif (type_data == "t_maxmin"):
            smax = smin = 0
            for i in range(self.len):
                for j in range(self.len):
                    smax += sign(self.dataframe.iloc[j,1] - self.dataframe.iloc[i, 1])
                    smin += sign(self.dataframe.iloc[j,2] - self.dataframe.iloc[i, 2])
            return (smax, p_value(smax)), (smin, p_value(smin))





import os
import numpy as np
import shutil
from datetime import datetime, timedelta
from zipfile import ZipFile
import requests
from matplotlib import pyplot as plt

class Inemet:
    def __init__(self, path, encoding='utf-8', sep=';', type=None, list=False):

        """
        :param path:
        :param encoding:
        :param sep:
        :param type: Tipo de dados que tem no arquivo t_maxmin: Temperatura máxima e mínima
        """

        def linha_inicio_df(arquivo):
            """Abre o arquivo porém pula linhas em branco que por padrão são 4
            No inicio a primeira linha é transformada em head. Portanto, somamos+4+1"""
            df = pd.read_csv(arquivo, sep=';', on_bad_lines='skip',
                             encoding='latin-1')
            return len(df) + 2

        def find_cod(df):
            """
            Encontra o código da estação no dataframe
            """
            line_codigo = 0

            while True:
                temp = str(df.loc[line_codigo]).split()
                # se código aparece na linha, sabemos que é esta
                if 'Codigo' in temp:
                    ind = temp.index('Codigo')
                    temp = temp[ind + 2]
                    return temp
                line_codigo += 1

        def type_data(x):
            if x in ("t_maxmin", "precipitation"):  #If user say the variable
                return x
            df = pd.read_csv(path, skiprows=linha_inicio_df(path), nrows=50, on_bad_lines='skip')
            temp = str.lower(''.join(df.columns))
            if 'temperatura' in temp:
                if "minima" and 'maxima' in temp:

                    del df
                    del temp
                    return 't_maxmin'
                del df
                del temp
                return 't_med'
            if 'precipitacao':
                del df
                del temp
                return 'precipitation'

        def date(linha):

            """
                        Encontra a altura da estação no dataframe
                        """
            import datetime
            df = self.dataframe
            df = df.iloc[linha, 0]
            df = datetime.datetime.strptime(df, '%Y-%m-%d')
            return df

        def coords(path):
            df = pd.read_csv(path, nrows=linha_inicio_df(path), index_col=False,
                                           on_bad_lines='skip', encoding='latin-1')
            line_codigo = 0
            cord = {'lat': '', 'lon': ''}
            while True:
                temp = str(df.loc[line_codigo]).split()
                # se código aparece na linha, sabemos que é esta
                if 'Latitude:' in temp:
                    ind = temp.index('Latitude:')
                    temp = temp[ind+1]
                    cord['lat'] = temp

                    line_codigo += 1
                    temp = str(df.loc[line_codigo]).split()
                    ind = temp.index('Longitude:')
                    temp = temp[ind + 1]
                    cord['lon'] = temp
                    return cord
                line_codigo += 1

        def alt(path):
            df = pd.read_csv(path, nrows=linha_inicio_df(path), index_col=False,
                             on_bad_lines='skip', encoding='latin-1')
            line_codigo = 0
            while True:
                temp = str(df.loc[line_codigo]).split()
                # If the code appear at line
                if 'Altitude:' in temp:
                    ind = temp.index('Altitude:')
                    temp = temp[ind + 1]
                    return temp
                line_codigo += 1

        def situacao(path):
            df = pd.read_csv(path, nrows=linha_inicio_df(path), index_col=False,
                             on_bad_lines='skip', encoding='latin-1')
            line_codigo = 0
            while True:
                temp = str(df.loc[line_codigo]).split()
                # se código aparece na linha, sabemos que é esta
                if 'Situacao:' in temp:
                    ind = temp.index('Situacao:')
                    temp = temp[ind + 1]
                    return temp
                line_codigo += 1

        def cidade(path):
            df = pd.read_csv(path, nrows=linha_inicio_df(path), index_col=False,
                             on_bad_lines='skip', encoding='latin-1')
            line_codigo = 0
            while True:
                temp = str(df.loc[line_codigo]).split()
                # se código aparece na linha, sabemos que é esta
                if 'Nome:' in temp:
                    ind = temp.index('Nome:')
                    temp = temp[ind + 1]
                    return temp
                line_codigo += 1

        def dataframe():
            if self.type_data == 't_maxmin':
                df = pd.read_csv(path, encoding=encoding,
                                         index_col=False, sep=sep, skiprows=linha_inicio_df(path))
                df.columns = ['Data', 'max', 'min','']
                df = df.drop(columns='')
                return df
            elif self.type_data == 'precipitation':
                df = pd.read_csv(path, encoding=encoding,
                                 index_col=False, sep=sep, skiprows=linha_inicio_df(path))
                df.columns = ['Data', 'pr', '']
                df = df.drop(columns='')
                return df
            elif self.type_data == 't_med':
                df = pd.read_csv(path, encoding=encoding,
                                 index_col=False, sep=sep, skiprows=linha_inicio_df(path))
                df.columns = ['Data', 'med', '']
                df = df.drop(columns='')

                return df

        import pandas as pd
        if not list:
            self.type_data = type_data(type)
            self.dataframe = dataframe()
            self.code = find_cod(pd.read_csv(path, nrows=linha_inicio_df(path), index_col=False,
                                               on_bad_lines='skip', encoding='latin-1'))
            self.startdate = date(0)
            self.enddate = date(len(self.dataframe)-1)
            self.lat, self.lon = coords(path)['lat'], coords(path)['lon']
            self.altitude = alt(path)
            self.situacao = situacao(path)
            self.type= 'original'
            self.city = cidade(path)
            self.columns = self.dataframe.columns
            self.len = len(self.dataframe)
            self.list = False
        else:
            self.path = path
            self.list = True

    def show(self):
        print(self.dataframe)

    def empty_data(self, type='absolute'):
        import math

        if self.type != "format1":
            self.format1(comma_to_dot=True)
        # if data is temperature
        if self.type_data == 't_maxmin':
            cmax = np.sum(np.isnan(self.dataframe.iloc[:,1])) # Nan of max temperature
            cmin = np.sum(np.isnan(self.dataframe.iloc[:,2])) #Nan of min temperature
            if type == 'relative':
                return ((cmax / len(self.dataframe)) * 100, (cmin/len(self.dataframe))*100)
            elif type == 'absolute':
                return cmax, cmin
            else:
                raise "select one of two possibles types 'relative' or 'absolute'"

        # se estamos tratando de precipitação
        elif self.type_data == 'precipitation':
            c = np.sum(np.isnan(self.dataframe.iloc[:, 1]))
            if type == 'relative':
                return round((c / len(self.dataframe)) * 100)
            elif type == 'absolute':
                return round(c)
            else:
                raise "select one of two possibles types 'relative' or 'absolute'"
    def report(self, path):
        """
        Save a txt file with importants informations

        """
        self.format1()
        if (self.type_data == "t_maxmin"):
            with open(path, 'w') as f:
                f.write(f"Station {self.city} {self.code}\n")
                f.write(f"Empty data of {self.type_data} : {self.empty_data()}\n")
                f.write(f"Max value of max temperature: {np.max(self.dataframe.iloc[:, 1])}\n")
                f.write(f"Max value of min temperature: {np.max(self.dataframe.iloc[:, 2])}\n")
                v= self.dataframe.iloc[np.argmax(self.dataframe.iloc[:, 1]),0]
                f.write(f"Date of max value of max temperature: {v.day}/{v.month}/{v.year}\n")
                v= self.dataframe.iloc[np.argmax(self.dataframe.iloc[:, 2]),0]
                f.write(f"Date of max value of min temperature: {v.day}/{v.month}/{v.year}\n")
                f.write(f"Min value of max temperature: {np.min(self.dataframe.iloc[:, 1])}\n")
                f.write(f"Min value of min temperature: {np.min(self.dataframe.iloc[:, 2])}\n")
                v= self.dataframe.iloc[np.argmin(self.dataframe.iloc[:, 1]),0]
                f.write(f"Date of min value of max temperature: {v.day}/{v.month}/{v.year}\n")
                v= self.dataframe.iloc[np.argmin(self.dataframe.iloc[:, 2]),0]
                f.write(f"Date of min value of min temperature: {v.day}/{v.month}/{v.year}\n")
                f.close()
        elif (self.type_data == "precipitation"):
            with open(path, 'w') as f:
                f.write(f"Station {self.city} {self.code}\n")
                f.write(f"Empty data of {self.type_data}: {self.empty_data()}\n")
                f.write(f"Max value: {np.max(self.dataframe.iloc[:, 1])}\n")
                v= self.dataframe.iloc[np.argmax(self.dataframe.iloc[:, 1]),0]
                f.write(f"Date of max value: {v.day}/{v.month}/{v.year}\n")
                f.write(f"Min value: {np.min(self.dataframe.iloc[:, 1])}\n")
                v= self.dataframe.iloc[np.argmin(self.dataframe.iloc[:, 1]),0]
                f.write(f"Date of min value: {v.day}/{v.month}/{v.year}\n")
                f.close()
    def report_mf(self, path_in, path_out, var="precipitation", years=(0, 0)):
            """
            Save a txt file with imports informations of multiples files

            :param path_in: folder when are the files, only the files
            :param path_out: Folder when the output will be save
            """
            import pandas as pd
            files = os.listdir(path_in)
            if ( var == "t_maxmin" ):
                df = pd.DataFrame(columns=["Code of station", "Period", "Missing data Tmax","Missing data Tmin", "Max value Tmax","Max value Tmin", "Date of max Tmax", "Date max Tmin"])
                df["Code of station"] = np.arange(len(files))
                for i in range(len(files)):
                    ana = Inemet(path_in+"/"+files[i])
                    if (years == (0,0)):
                        ana.format1()
                    else:
                        ana.format1(years=years)
                    df.iloc[i, 0] = ana.code #Salva o código da estação 
                    df.iloc[i, 1] = f"{ana.startdate.day}/{ana.startdate.month}/{ana.startdate.year} - {ana.enddate.day}/{ana.enddate.day}/{ana.enddate.year}" #Salva o período da estação 
                    df.iloc[i, 2], df.iloc[i, 3] = ana.empty_data() #Salva os dados faltantes 
                    df.iloc[i, 4] = np.max(ana.dataframe.iloc[:, 1]) #Salva o valor máximo 
                    df.iloc[i, 5] = np.max(ana.dataframe.iloc[:, 2]) #Salva o valor máximo 
                    df.iloc[i, 6] = ana.dataframe.iloc[np.argmax(ana.dataframe.iloc[:, 1]), 0]
                    df.iloc[i, 7] = ana.dataframe.iloc[np.argmax(ana.dataframe.iloc[:, 2]), 0]
                df.to_csv(path_out)
             
    def plot(self):
        """

        :param title:
        :param xlabel:
        :param ylabel:
        :return:
        """
        from matplotlib import pyplot as plt
        from datetime import datetime
        colum = self.dataframe.columns
        if self.type_data == 'temperatura':
            y = self.dataframe[colum[1]]
            x = self.dataframe[colum[0]]
            temp = []
            for k in x:
                temp.append(datetime.strptime(k, '%Y-%m-%d'))
            x = temp
            plt.plot(x, y, linewidth=0.1)
            plt.show()

        elif self.type_data == 'precipitation':
            y = self.dataframe[colum[1]]
            x = self.dataframe[colum[0]]
            temp = []
            for k in x:
                temp.append(datetime.strptime(k, '%Y-%m-%d'))
            x = temp
            plt.bar(x, y)
            plt.tight_layout()

            plt.show()

    def get_year(self, year, with_x=False, return_x0=False):
        """
        From array muti-year, return a year specific
        :param year: can by a Int, or list of Int
        :param with_x:
        :param return_x0: Return the index init
        :return:
        """
        import pandas as pd
        self.format1()
        if isinstance(year, int):  # if is only a year
            try:
                a = pd.to_datetime(
                    self.dataframe[
                        'time']).dt.year.to_numpy() == year  # array with true or false if the year is that we want
            except:
                raise f"The Data don't have the year {year}"
            a = np.where(a == True)[0]  # Return array with index of True and correct the list format
            if with_x:
                return self.dataframe['time'][a], self.dataframe['pr'][a]
            elif return_x0:
                return a[0], self.dataframe['time'][a]
            return self.dataframe['pr'][a]
        else:  # is many years
            xf = pd.Series([])
            yf = np.array([])
            for i in year:
                try:
                    a = pd.to_datetime(
                        self.dataframe[
                            'time']).dt.year.to_numpy() == i  # array with true or false if the year is that we want
                except:
                    raise f"The Data don't have the year {i}"

                a = np.where(a == True)[0]  # Return array with index of True and correct the list format
                xf = pd.concat([xf, self.dataframe['time'][a]])
                yf = np.concatenate([yf, self.dataframe['pr'][a].to_numpy()])
            if with_x:
                return xf, yf
            elif return_x0:
                raise "This function exist only to one year"
            return yf
    def format1(self, comma_to_dot=True, grow=True, years=(0, 0)):
        """
        We change the file format to
        1) put date that can be jumped
        2) organize the file from to smaler to larger based in time 
        3) Change the ',' to '.'
        4) Remove same values from file
        5) Change the name of time variable to time
        6) padronize a file to work with Ana, Inemet, Merge and etc
        7) have option to select a series of year start and year end. The date that not exist, are created and set NaN

        The out file is in format 

                        time    pr
                        1997-07-01  0.1
                        1997-07-02  0.3
                        ....
                        ...
                        ...
                        2020-01-01  0.3
                        2020-01-02  0.0
 
        """

        import pandas as pd
        if self.type_data == 'precipitation':
            import datetime
            # if change to format1 and complete the years
            if self.type != 'format1' and years != (0, 0):
                from datetime import datetime

                # organize the file from smaller to larger
                df = self.dataframe
                df['Data'] = pd.to_datetime(df['Data'], format="%Y-%m-%d")
                df = df.drop_duplicates(subset='Data', keep="last")
                df = df.rename(columns={"Data": 'time'})

                if comma_to_dot:
                    df = df.replace({',': '.'}, regex=True)
                    df = df.reset_index()
                    df = df.drop(columns='index')
                    self.dataframe = df
                if not grow:
                    self.dataframe = df.sort_values(by=['time'], ascending=True)

                a = pd.date_range(start=datetime.date(self.startdate),
                                     end=self.enddate)
                a=a[:-1]
                data = pd.to_datetime(df["time"])
                pr = np.array(df['pr'])
                for i in range(len(a)):
                    if data[i] != a[i]:
                        t = pd.date_range(data[i - 1], data[i])[1:-1]
                        data = np.concatenate((data[:i], t, data[i:]))
                        pr = np.concatenate((pr[:i+1], (len(t)-2) * [np.NaN], pr[i-1:]))
                data = pd.date_range(start=datetime.date(self.startdate),
                                     end=self.enddate)  # change data to numpy datetime64
                del a

                # chech if have date jump
                a = pd.date_range(start=str(years[0]), end=str(years[1]), freq='D')
                a=a[:-1]
                pr_def = np.full(len(a), np.nan)
                for i in range(len(data)):
                    ind = np.argwhere(data[i] == a)
                    pr_def[ind] = pr[i]

                df = pd.DataFrame(list(zip(a, pr_def)), columns=['time', 'pr'])

                # atualize the format and len
                self.type = "format1"
                df = df[(df.iloc[:, 0].dt.year <= years[1])]
                self.dataframe = df[df.iloc[:, 0].dt.year >= years[0]]
                self.len = len(self.dataframe)
                self.enddate = df.iloc[-1,0] 
                self.startdate = df.iloc[0, 0]


            elif self.type != 'format1':
                from datetime import datetime

                # organize the file from smaler to larger
                df = self.dataframe
                df['Data'] = pd.to_datetime(df['Data'], format="%Y-%m-%d")
                df = df.drop_duplicates(subset='Data', keep="last")
                df = df.rename(columns={"Data":'time'})
                if comma_to_dot:
                    df = df.replace({',': '.'}, regex=True)
                    df = df.reset_index()
                    df = df.drop(columns='index')
                    self.dataframe = df
                if not grow:
                    self.dataframe = df.sort_values(by=['time'], ascending=True)

                colum = self.dataframe.columns
                data = pd.to_datetime(self.dataframe[colum[0]])
                pr = self.dataframe[colum[1]].to_numpy()

                # chech if have date jump
                a = np.arange(self.startdate, self.enddate, timedelta(days=1)).astype(datetime)
                for i in range(len(a)):
                    if data[i] != a[i]:
                        t = pd.date_range(data[i - 1], data[i])[1:-1]
                        data = np.concatenate((data[:i], t, data[i:]))
                        pr = np.concatenate((pr[:i], len(t) * [np.NaN], pr[i:]))
                data = pd.date_range(start=datetime.date(self.startdate),
                                     end=self.enddate)  # change data to numpy datetime64

                df = pd.DataFrame(list(zip(data, pr)), columns=['time', 'pr'])
                self.dataframe = df

        elif self.type_data == 't_maxmin':
            from datetime import datetime

            # organize the date from smallest to bigest
            df = self.dataframe
            df['Data'] = pd.to_datetime(df['Data'], format="%Y-%m-%d")
            df = df.drop_duplicates(subset='Data', keep="last")

            # comma to point
            if comma_to_dot:
                df = df.replace({',': '.'}, regex=True)
                df = df.reset_index()
                df = df.drop(columns='index')
                df = df.rename(columns={"Data": 'time'})
                self.dataframe = df

            if not grow:
                df = df.rename(columns={'Data': 'Data'})
                self.dataframe = df.sort_values(by=['Data'], ascending=True)
                self.dataframe = self.dataframe.rename(columns={"Data": 'time'})
            # if change to format1 and complete the years
            if self.type != 'format1' and years != (0, 0):
                colum = self.dataframe.columns
                data = pd.to_datetime(self.dataframe[colum[0]])
                max = np.array(self.dataframe[colum[1]].to_numpy())
                min = np.array(self.dataframe[colum[2]].to_numpy())

                # chech if have date jump
                a = pd.date_range(datetime.date(self.startdate), self.enddate)

                # chech if have date jump
                for i in range(len(a)):
                    if data[i] != a[i]:
                        t = pd.date_range(data[i - 1], data[i])[1:-1]
                        data = np.concatenate((data[:i], t, data[i:]))
                        max = np.concatenate((max[:i], len(t) * [np.NaN], max[i:]))
                        min = np.concatenate((min[:i], len(t) * [np.NaN], min[i:]))

                a = pd.date_range(start=str(years[0]), end=str(years[1]), freq='D')
                a=a[:-1]
                max_def = np.full(len(a), np.nan)
                min_def = np.full(len(a), np.nan)
                for i in range(len(data)):
                    ind = np.argwhere(data[i] == a)
                    max_def[ind] = max[i]
                    min_def[ind] = min[i]
                # atualize the format and len
                self.dataframe = pd.DataFrame(list(zip(a, max, min)), columns=['time', 'max', "min"])
                self.type = "format1"
                self.len = len(self.dataframe)
                self.enddate = df.iloc[-1,0] 
                self.startdate = df.iloc[0, 0]

            elif self.type != "format1":
                colum = self.dataframe.columns
                data = pd.to_datetime(self.dataframe[colum[0]])
                max = self.dataframe[colum[1]].to_numpy()
                min = self.dataframe[colum[2]].to_numpy()
                a = pd.date_range(start=datetime.date(self.startdate),
                                     end=self.enddate)
                a=a[:-1]


                # chech if have date jump
                for i in range(len(a)):
                    if data[i] != a[i]:
                        t = pd.date_range(data[i - 1], data[i])[1:-1]
                        data = np.concatenate((data[:i], t, data[i:]))
                        max = np.concatenate((max[:i], len(t)*[np.NaN], max[i:]))
                        min = np.concatenate((min[:i], len(t)*[np.NaN], min[i:]))

                data = pd.date_range(start=datetime.date(self.startdate),
                                     end=self.enddate)  # change data to numpy datetime64

                # atualize the format and len
                self.type = "format1"
                self.len = len(self.dataframe)
                self.dataframe = pd.DataFrame(list(zip(data, max, min)), columns=['time', 'max', 'min'])
        elif self.type_data == 't_med':
            from datetime import datetime
        
            # Organizando o arquivo pela datas do menor até o maior
            df = self.dataframe
            df['Data'] = pd.to_datetime(df['Data'], format="%Y-%m-%d")
            df = df.drop_duplicates(subset='Data', keep="last")
            if comma_to_dot:
                df = df.replace({',': '.'}, regex=True)
                df = df.rename(columns={'Data': 'time'})
                df = df.reset_index()
                df = df.drop(columns='index')
                self.dataframe = df
            if not grow:
                df = df.rename(columns={'Data': 'time'})
                self.dataframe = df.sort_values(by=['time'], ascending=True)

            colum = self.dataframe.columns
            data = self.dataframe[colum[0]].to_numpy()
            med = self.dataframe[colum[1]].to_numpy()
            date = pd.date_range(start=datetime.date(self.startdate),
                                 end=self.enddate)  # change data to numpy datetime64
            # chech if have date jump
            for i in range(len(date)):
                try:
                    while np.datetime64(data[i]) != np.datetime64(date[i]):
                        data = np.insert(data, i, data[i - 1] + timedelta(days=1))
                        med = np.insert(med, i, np.NaN)
                except:
                    continue

            if self.type == "format1" and years != (0,0):
                self.dataframe = pd.DataFrame(list(zip(date, med)), columns=['time', "med"])

                # atualize the format and len
                self.len = len(self.dataframe)
            elif self.type == "format":
                self.dataframe = pd.DataFrame(list(zip(data, med)), columns=['time', 'med'])

                # atualize the format and len
                self.type = "format1"
                self.len = len(self.dataframe)

        self.type = 'format1'

    def get_month(self, month, year, with_x=False):
        """
        From array muti-year, return a month specific
        :param month: The month to select as Int, or list of month to get
        :param year: The year to select
        :with_x: return the data, True or Not
        :return: Array with data of month select
        """
        if self.type != "format1":
            self.format1()
        #x0 is the index wen start the year that we want
        x0, time = self.get_year(year, return_x0=True)
        if isinstance(month, int):
            try:
                a = time.dt.month.to_numpy() == month  # array with true or false if the year is that we want
            except:
                raise f"The data has't the month {month}"
            a = np.where(a == True)[0]  # Return array with index of True and correct the list format
            a += x0
            if with_x:
                return self.dataframe['time'][a], self.dataframe['pr'][a]
            return self.dataframe['pr'][a]
        else:
            xf = pd.Series([])
            yf = np.array([])
            for i in month:
                try:
                    a = time.dt.month.to_numpy() == i  # array with true or false if the year is that we want
                except:
                    raise f"The data has't the month {i}"
                a = np.where(a == True)[0]  # Return array with index of True and correct the list format
                a += x0
                xf = pd.concat([xf, self.dataframe['time'][a]])
                yf = np.concatenate([yf, self.dataframe['pr'][a].to_numpy()])
            if with_x:
                return xf, yf
            return yf

    #Convert to another version
    def to_netcdf4(self, local=''):
        import xarray as xr
        if self.type_data == 'precipitacao':
            self.format1(grow=False)
            df = self.dataframe

            df = df.rename(columns={'precipitação': 'pr'})  # verificar se é este o nome
            df = df.rename(columns={'Data': 'time'})

            # criando latitude e longitude com valor (0,0)
            lat = [float(self.lat)] * len(df['pr'])
            lon = [float(self.lon)] * len(df['pr'])
            df['lat'] = lat
            df['lon'] = lon
            # abrindo o df com o xarray
            ds = df.set_index(['time', 'lat', 'lon']).to_xarray()

            ds = xr.Dataset(ds)
            # add variable attribute metadata
            ds.attrs['lat'] = 'Units: °'
            ds.attrs['lon'] = 'Units: °'
            ds.attrs['time'] = 'Daily Data'

            return ds.to_netcdf(local+f'/Dados{self.codigo}_D_{self.startdate.year}_{self.enddate.year}.nc')

        elif self.type_data == 't_maxmin':
            # remove colunas em branco
            self.format1()
            df = self.dataframe.drop(['TEMPERATURA MINIMA, DIARIA(°C)'], axis=1)

            df = df.rename(columns={'TEMPERATURA MAXIMA, DIARIA(°C)': 'tas'})  # verificar se é este o nome
            df = df.rename(columns={'Data Medicao': 'time'})
            # pr para numero
            temp = []
            for k in df['tas']:
                temp.append(float(k))
            df['tas'] = temp[:]

            # criando latitude e longitude com valor (0,0)
            lat = [float(self.coord['latitude'])] * len(df['tas'])
            lon = [float(self.coord['longitude'])] * len(df['tas'])
            df['lat'] = lat
            df['lon'] = lon
            # abrindo o df com o xarray
            ds = df.set_index(['time', 'lat', 'lon']).to_xarray()

            ds = xr.Dataset(ds)
            # add variable attribute metadata
            ds.attrs['lat'] = 'Units: °'
            ds.attrs['lon'] = 'Units: °'
            ds.attrs['time'] = 'Data diária'

            ds.to_netcdf(local + f'/Dados{self.codigo}_D_{self.startdate.year}_{self.enddate.year}_max.nc')

            # Calcular mínimo
            df = self.dataframe.drop(['TEMPERATURA MAXIMA, DIARIA(°C)'], axis=1)

            df = df.rename(columns={'TEMPERATURA MINIMA, DIARIA(°C)': 'tas'})  # verificar se é este o nome
            df = df.rename(columns={'Data Medicao': 'time'})
            # pr para numero
            temp = []
            for k in df['tas']:
                temp.append(float(k))
            df['tas'] = temp[:]

            # criando latitude e longitude com valor (0,0)
            lat = [float(self.coord['latitude'])] * len(df['tas'])
            lon = [float(self.coord['longitude'])] * len(df['tas'])
            df['lat'] = lat
            df['lon'] = lon
            # abrindo o df com o xarray
            ds = df.set_index(['time', 'lat', 'lon']).to_xarray()

            ds = xr.Dataset(ds)
            # add variable attribute metadata
            ds.attrs['lat'] = 'Units: °'
            ds.attrs['lon'] = 'Units: °'
            ds.attrs['time'] = 'Data diária'

            ds.to_netcdf(local + f'/Dados{self.codigo}_D_{self.startdate.year}_{self.enddate.year}_min.nc')

    def to_csv(self, local=""):
        self.dataframe.to_csv(local)

    def date(self, linha):

        """
                    Encontra a altura da estação no dataframe
                    """
        import datetime
        df = self.dataframe
        df = df.iloc[linha, 0]
        df = datetime.datetime.strptime(df, '%Y-%m-%d')
        return df

    def close(self):
        import gc
        gc.collect()

    def ByYear(self, year, var):
        """
        Of a dataframe, return a segment based in one year
        :param year: int, represent the year of segment
        :param var: str, is the name of variable you want data
        :return: array with variable data of one year
        """
        # research the year in file
        inicio, final = 0, 0
        for linha in range(self.len):
            l = linha
            if self.date(linha).year == year:
                inicio = linha  #start line
                while self.date(l).year == year and l < self.len-1:
                    l += 1
                final = l
                break
        return np.array(self.dataframe[f'{var}'][inicio:final])

    # Climate index
    def rx5day(self, valor=50, retornar='highest_five_day_precipitation_amount_per_time_period', selbyyear=0):
        import datetime
        '''
        :param valor:
        :param retornar:Opções para retornar: highest_five_day_precipitation_amount_per_time_period, que é uma lista 0 é o numero e 1 é o mês
        ; number_of_5day_heavy_precipitation_periods_per_time_period
        :return:
        '''

        # deixa apenas os dados de precipitação
        ds = self.dataframe
        number_of_5day_heavy_precipitation_periods_per_time_period = 0
        lista = []
        maior = {0: 0, 1: 0}
        cont = temp = 0
        for linha in range(len(self.dataframe)):
            # percorre as colunas
            # verifica se a data existe, se não, temos um erro
            data = self.date(linha)
            if data.year == selbyyear:
                #
                if cont < 5:
                    temp += float(ds.iloc[linha, 1])
                    cont += 1
                # se passou 5 dias, armazenamos na lista
                elif cont == 5:
                    lista.append({'num': temp, 'month': (data+datetime.timedelta(days=-5))})
                    cont = 0
                    temp = 0

        # depois de criar uma lista, iremos percorre ela em busca dos valores
        for k in lista:
            # se teve um excedente de chuva acima de um valor
            if k['num'] > valor:
                if k['num'] > maior[0]:
                    maior = {0: k['num'], 1: k['month']}
                number_of_5day_heavy_precipitation_periods_per_time_period += 1

        if retornar == 'highest_five_day_precipitation_amount_per_time_period':
            return maior
        elif retornar == 'number_of_5day_heavy_precipitation_periods_per_time_period':
            return number_of_5day_heavy_precipitation_periods_per_time_period

    def tx(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        return ind.tx(with_x=with_x, type_data=self.type_data, var='max')

    def tn(self, with_x=False):
        from AgenciBr.indice import Indice
        ind = Indice(self.dataframe)
        return ind.tn(with_x=with_x, type_data=self.type_data)

    def tn90p(self, with_x=False):
        from AgenciBr.indice import Indice
        ind = Indice(self.dataframe)
        return ind.tn90p(with_x=with_x)

    def tx90p(self, with_x=False):
        from AgenciBr.indice import Indice
        ind = Indice(self.dataframe)
        ind = ind.tx90p(with_x=with_x, type_data=self.type_data)
        return ind

    #Download
    def download(self, path,  year_start=2000, year_end=datetime.now().year, file_save='zip', list_station='all'):
        """
        Download the data from site https://portal.inmet.gov.br/dadoshistoricos
        and change automatic to a model that you need

        :param path: The path to save the file
        :param year_start: The year that start the download
        :param year_end:
        :param file_save: The type of out file
        :param list_station: set a list of specify stations or all
        :return:
        """
        os.mkdir(f"{path}/tempo")
        for i in np.arange(year_start, year_end+1):  # list the years
            resposta = requests.get(f"https://portal.inmet.gov.br/uploads/dadoshistoricos/{i}.zip")
            # Acessa o conteúdo e salva em um arquivo
            if resposta.status_code == requests.codes.OK:
                with open(f"{path}/tempo/dadoshistóricos_{i}.zip", 'wb') as novo:
                    novo.write(resposta.content)
            else:
                resposta.raise_for_status()

        # if out file is zip file
        if file_save == 'zip':
            if list_station == 'all':
                for i in os.listdir(path+f'/tempo'):  # remove the temporary diretory and promove the files
                    shutil.move(path+f'/tempo/{i}', path+f"/{i}")
                os.rmdir(path+f"/tempo")
        # if out file is stract to directory
        if file_save == 'dir':
            if list_station == 'all':
                for i in os.listdir(path+f'/tempo'):  # remove the temporary diretory and promove the files
                    shutil.move(path+f'/tempo/{i}', path+f"/{i}")
                os.rmdir(path+f"/tempo")

                for k in os.listdir(path):
                    if int(k[-8:-4]) < 2020:
                        z = ZipFile(path+"/"+k, 'r')
                        z.extractall(path)
                        z.close()
                        os.remove(f"{path}/{k}")
                    else:
                        z = ZipFile(path + "/" + k, 'r')
                        os.mkdir(f"{path}/{k[-8:-4]}")
                        z.extractall(f"{path}/{k[-8:-4]}")
                        z.close()
                        os.remove(f"{path}/{k}")
            else: #is a city
                for i in os.listdir(path + f'/tempo'):  # remove the temporary diretory and promove the files
                    shutil.move(path + f'/tempo/{i}', path + f"/{i}")
                os.rmdir(path + f"/tempo")

                for k in os.listdir(path):
                    if int(k[-8:-4]) < 2020:
                        z = ZipFile(path + "/" + k, 'r')
                        z.extractall(path)
                        z.close()
                        os.remove(f"{path}/{k}")
                    else:
                        z = ZipFile(path + "/" + k, 'r')
                        os.mkdir(f"{path}/{k[-8:-4]}")
                        z.extractall(f"{path}/{k[-8:-4]}")
                        z.close()
                        os.remove(f"{path}/{k}")
                # remove the diferents
                for i in os.listdir(path):
                    city = list_station.upper()

                    for k in os.listdir(path+f"/{i}"):
                        if city not in k:
                            os.remove(f"{path}/{i}/{k}")

        # list functions

    def to_climatol(self, path, years, erro=10):
        import pandas as pd
        """
        :param path:
        :param years:
        :return:
        """
        if self.list == False:
            raise "This function need a list of stations, change list=True and see the documentation"
        else:
            import os
            posi = []
            data = np.array([])
            c = 0
            for i, k in enumerate(os.listdir(self.path)):
                    a = Inemet(self.path + "/" + k)
                    a.format1(years=years)
                    if a.empty_data()< erro:
                        if c == 0:
                            data = np.array([a.dataframe["pr"]])
                            posi.append([a.lat, a.lon, a.altitude, f"{a.code}", f"Ana-{a.code}"])
                            c+=1
                        else:
                            data = np.append(data, [a.dataframe["pr"]], axis=0)
                            posi.append([a.lat, a.lon, a.altitude, f"{a.code}", f"Inemet-{a.code}"])

            a = pd.DataFrame(np.transpose(data))
            b = pd.DataFrame(posi)

            # Before to save, climatol find empty place with some word in string "nan". Because this, we change to str
            a = a.astype(str)
            b = b.astype(str)
            a.to_csv(path + f"/pr_{years[0]}-{years[1]}.dat", index=False, header=False, sep=" ")
            b.to_csv(path + f"/pr_{years[0]}-{years[1]}.est", index=False, header=False, sep=" ")

        with open("inemet/climatol.r", "w") as climatol:
            climatol.write(f"""
            library(climatol)

            #------------------------------------------------------
            setwd("{self.path}")

            dat <- as.matrix(read.table("pr_{years[0]}-{years[1]}.dat"))
            write(dat, "Ttest_{years[0]}-{years[1]}.dat")

            homogen('pr', {years[0]}, {years[1]}, expl = TRUE)

            """)
            climatol.close()
        os.system("Rscript inemet/climatol.r")



import os
import xarray as xr
class Merge:
    from datetime import datetime

    def __init__(self, arquivo=None):
        if arquivo!= None:
            self.dataset =arquivo
    
    def download(self, path_save, year_start=2000, year_end=datetime.now().year, date=None, to="original"):
        import datetime
        import requests

        total = (year_end-year_start)*360
        cont=0
        def download_file(url, endereco, nome):
            # acessa o site
            resposta = requests.get(url)
            # Acessa o conteúdo e salva em um arquivo
            if resposta.status_code == requests.codes.OK:
                with open(endereco + f'/{nome}.grib2', 'wb') as novo:
                    novo.write(resposta.content)
            else:
                resposta.raise_for_status()
        if ( date != None):
            if ( date.month < 10):
                if ( date.day < 10):
                    download_file(
                    url=f'http://ftp.cptec.inpe.br/modelos/tempo/MERGE/GPM/DAILY/{date.year}/0{date.month}/'
                    f'MERGE_CPTEC_{date.year}0{date.month}0{date.day}.grib2',
                    endereco=f'{path_save}',
                    nome=f'MERGE_CPTEC_{date.year}0{date.month}0{date.day}')
                    if to == "netcdf":
                        df = xr.open_dataset(f"{path_save}/MERGE_CPTEC_{date.year}0{date.month}0{date.day}.grib2", engine="cfgrib")
                        df.to_netcdf(f"{path_save}/MERGE_CPTEC_{date.year}0{date.month}0{date.day}.nc")
                        os.remove(f"{path_save}/MERGE_CPTEC_{date.year}0{date.month}0{date.day}.grib2")
                        os.system(f"rm {path_save}/*.idx")


                else:
                    download_file(
                    url=f'http://ftp.cptec.inpe.br/modelos/tempo/MERGE/GPM/DAILY/{date.year}/0{date.month}/'
                    f'MERGE_CPTEC_{date.year}0{date.month}{date.day}.grib2',
                    endereco=f'{path_save}',
                    nome=f'MERGE_CPTEC_{date.year}0{date.month}{date.day}')
                    if to == "netcdf":
                        df = xr.open_dataset(f"{path_save}/MERGE_CPTEC_{date.year}0{date.month}{date.day}.grib2", engine="cfgrib")
                        df.to_netcdf(f"{path_save}/MERGE_CPTEC_{date.year}0{date.month}{date.day}.nc")
                        os.remove(f"{path_save}/MERGE_CPTEC_{date.year}0{date.month}{date.day}.grib2")
                        os.system(f"rm {path_save}/*.idx")


            else:
                if ( date.day < 10):
                    download_file(
                    url=f'http://ftp.cptec.inpe.br/modelos/tempo/MERGE/GPM/DAILY/{date.year}/{date.month}/'
                    f'MERGE_CPTEC_{date.year}{date.month}0{date.day}.grib2',
                    endereco=f'{path_save}',
                    nome=f'MERGE_CPTEC_{date.year}{date.month}0{date.day}')
                    if to == "netcdf":
                        df = xr.open_dataset(f"{path_save}/MERGE_CPTEC_{date.year}{date.month}0{date.day}.grib2", engine="cfgrib")
                        df.to_netcdf(f"{path_save}/MERGE_CPTEC_{date.year}{date.month}0{date.day}.nc")
                        os.remove(f"{path_save}/MERGE_CPTEC_{date.year}{date.month}0{date.day}.grib2")
                        os.system(f"rm {path_save}/*.idx")
                else:
                    download_file(
                    url=f'http://ftp.cptec.inpe.br/modelos/tempo/MERGE/GPM/DAILY/{date.year}/{date.month}/'
                    f'MERGE_CPTEC_{date.year}{date.month}{date.day}.grib2',
                    endereco=f'{path_save}',
                    nome=f'MERGE_CPTEC_{date.year}{date.month}{date.day}')
                    if to == "netcdf":
                        df = xr.open_dataset(f"{path_save}/MERGE_CPTEC_{date.year}{date.month}{date.day}.grib2", engine="cfgrib")
                        df.to_netcdf(f"{path_save}/MERGE_CPTEC_{date.year}{date.month}{date.day}.nc")
                        os.remove(f"{path_save}/MERGE_CPTEC_{date.year}{date.month}{date.day}.grib2")
                        os.system(f"rm {path_save}/*.idx")
        
        else:
            #all the cases name of file to download
            for ano in range(year_start, year_end):
                for mes in range(1, 13):
                    for dia in range(1, 32):
                        # Criaremos um arquivo datetime com a data dos loops, se não existir, ignora
                        try:
                            # Como o padrão do site é "06" e não "6" ára mês e dia, são conversões
                            if ano == 2000 and mes < 6:
                                break
                            time = datetime.datetime(ano, mes, dia)
                            if time.day < 10:  # adiciona o 0 na frente do marcador mês
                                if time.month < 10:  # Adicionamos 0 também
                                    download_file(
                                        url=f'http://ftp.cptec.inpe.br/modelos/tempo/MERGE/GPM/DAILY/{time.year}/0{time.month}/'
                                            f'MERGE_CPTEC_{time.year}0{time.month}0{time.day}.grib2',
                                        endereco=f'{path_save}',
                                        nome=f'MERGE_CPTEC_{time.year}0{time.month}0{time.day}')
                                else:

                                    download_file(
                                        url=f'http://ftp.cptec.inpe.br/modelos/tempo/MERGE/GPM/DAILY/{time.year}/{time.month}/'
                                            f'MERGE_CPTEC_{time.year}{time.month}0{time.day}.grib2',
                                        endereco=f'{path_save}',
                                        nome=f'MERGE_CPTEC_{time.year}{time.month}0{time.day}')
                            else:  # se pegamos dias maiores que 9
                                if time.month < 10:  # Adicionamos 0 também
                                    download_file(
                                        url=f'http://ftp.cptec.inpe.br/modelos/tempo/MERGE/GPM/DAILY/{time.year}/0{time.month}/'
                                            f'MERGE_CPTEC_{time.year}0{time.month}{time.day}.grib2',
                                        endereco=f'{path_save}',
                                        nome=f'MERGE_CPTEC_{time.year}0{time.month}{time.day}')
                                else:
                                    download_file(
                                        url=f'http://ftp.cptec.inpe.br/modelos/tempo/MERGE/GPM/DAILY/{time.year}/{time.month}/'
                                            f'MERGE_CPTEC_{time.year}{time.month}{time.day}.grib2',
                                        endereco=f'{path_save}',
                                        nome=f'MERGE_CPTEC_{time.year}{time.month}{time.day}')
                            cont+=1
                        except:
                            continue
                    print(f"{round((cont/total)*100)}% concluídos")
            if to == "netcdf":
                loc = os.listdir(path_save)
                for i in loc:
                    df = xr.open_dataset(f"{path_save}/{i}", engine="cfgrib")
                    df.to_netcdf(f"{path_save}/{i[:-6]}.nc")
                    os.remove(f"{path_save}/{i}")
                os.system(f"rm {path_save}/*.idx")
    def to_netcdf(self, path):
        import xarray as xr
        loc = os.listdir(path)
        for i in loc:
            df = xr.open_dataset(f"{path}/{i}", engine="cfgrib")
            df.to_netcdf(f"{path}/{i[:-6]}.nc")
            os.remove(f"{path}/{i}")
        os.system(f"rm {path}/*.idx")
        df = xr.open_mfdataset(f"{path}/*.nc")
        os.system(f"rm {path}/*.idx")

    def format2(self):
        self.dataset = self.dataset



'''
To understand the index:
https://csis.myclimateservice.eu/node/3527
https://www.climdex.org/learn/indices/#index-TX10p
http://etccdi.pacificclimate.org/list_27_indices.shtml

'''

from .Ana import Ana
from .Inemet import Inemet
from .Ideam import Ideam
from .Merge import Merge
from .Alexandre import Alexandre




import datetime
import geopandas as gpd
import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

class Ana:
    def __init__(self, arquivo, sep=';', index_col=False, list=False, encoding='latin-1',
                 on_bad_lines='skip', comma_to_dot=True, lat=None, lon=None):
        def find_cod(df):
            '''
            Encontra o código da estação no dataframe
            '''
            line_codigo = 0
            while True:
                temp = str(df.loc[line_codigo]).split()
                # se código aparece na linha, sabemos que é esta
                if 'Código' in temp:
                    ind = temp.index('Código')
                    temp = temp[ind + 2].split(':')
                    return int(temp[1])
                line_codigo += 1

        def linha_inicio_df(arquivo):
            '''Abre o arquivo porém pula linhas em branco que por padrão são 4
            No inicio a primeira linha é transformada em head. Portanto, somamos+4+1'''
            df = pd.read_csv(arquivo, sep=';',
                             encoding='latin-1', on_bad_lines='skip')
            return len(df) + 1 + 4

        def dataframe(path):
            df = pd.read_csv(path, skiprows=linha_inicio_df(path), sep=sep, encoding=encoding,
                             index_col=index_col, on_bad_lines=on_bad_lines)
            if comma_to_dot:
                temp = []
                
                df = df.drop_duplicates(subset='Data', keep="last")
                for k in df['Data']:
                    temp.append(datetime.datetime.strptime(k, '%d/%m/%Y'))
                df['Data'] = temp.copy()
                df = df.sort_values(by='Data')
                df = df.reset_index()
                return df.replace({',': '.'}, regex=True)
            else:
                temp = []
                for k in df['Data']:
                    temp.append(datetime.datetime.strptime(k, '%d/%m/%Y'))
                df['Data'] = temp.copy()
                df = df.sort_values(by='Data')
                df = df.reset_index()

                return df

        def get_startdate():
            if self.type == "format1":
                return self.dataframe["time"][0]
            return self.dataframe['Data'][0]
        def get_enddate():
            if self.type == "format1":
                return self.dataframe["time"][-1]

            if (self.dataframe['Data'][len(self.dataframe) - 1] + datetime.timedelta(days=30)).month == (self.dataframe['Data'][len(self.dataframe) - 1]).month:
                return self.dataframe['Data'][len(self.dataframe) - 1] + datetime.timedelta(days=30)
            return self.dataframe['Data'][len(self.dataframe) - 1] + datetime.timedelta(days=29)
        if not list:
            self.dataframe = dataframe(arquivo)
            self.type = "original"
            self.code = find_cod(pd.read_csv(arquivo, nrows=50, index_col=False,
                                               on_bad_lines='skip', encoding='latin-1'))
            self.fonte = pd.read_csv(arquivo, nrows=2, index_col=False,
                                     on_bad_lines='skip', encoding='latin-1')
            self.startdate = get_startdate()
            self.enddate = get_enddate()
            self.len = len(self.dataframe)
            self.type_data = 'pr'
            self.lat = lat
            self.lon = lon
            self.alt = None 
            self.name = None
            self.NumCity = None
            self.status = None
            self.city = None
            self.state = None
            self.list = list

        else:
            self.path = arquivo
            self.list = arquivo

    # internal functions
    def positions(self):
        if self.list == False:
            raise "This function exists only to list of stations"
        
        lat = np.array([])
        lon = np.array([])
        code = np.array([])
        for i in os.listdir(self.path):
            df = Ana(f"{self.path}/{i}")
            df.get_lonlat()
            lat = np.append(lat, df.lat)
            lon = np.append(lon, df.lon)
            code = np.append(code, df.code)

        df = pd.DataFrame({"code":code, "lat":lat, "lon":lon})
        return df
    def get_info(self):
        """Internal functions that get informations of the station"""
        inv = pd.read_csv("./inventario_ana.csv", sep=",", low_memory=False)
        a = inv[inv["Codigo"] == 466001]
        self.lat = float(a["Latitude"])
        self.lon = float(a["Longitude"])
        self.alt = float(a["Altitude"])
        self.name = str(a["Nome"])
    def only_mondata(self, list=False):
        """
        Remove all columns that is not referent of month precipitation and day, in original data from Agenci

        :param list:
        :return:
        """

        from datetime import datetime

        if list:
            list_temp = []
            for k in self.list:
                df = k
                lista_meses = ['Data', 'Chuva01', 'Chuva02', 'Chuva03', 'Chuva04', 'Chuva05',
                               'Chuva06', 'Chuva07', 'Chuva08', 'Chuva09', 'Chuva10',
                               'Chuva11', 'Chuva12', 'Chuva13', 'Chuva14', 'Chuva15',
                               'Chuva16', 'Chuva17', 'Chuva18', 'Chuva19', 'Chuva20',
                               'Chuva21', 'Chuva22', 'Chuva23', 'Chuva24', 'Chuva25',
                               'Chuva26', 'Chuva27', 'Chuva28', 'Chuva29', 'Chuva30',
                               'Chuva31']
                columns = df.columns
                for a in columns:
                    if a not in lista_meses:
                        del df[a]
                list_temp.append(df)

        elif list == False:
            df = self.dataframe
            lista_meses = ['Data']
            # adicionamos as colunas dos dados

            for k in range(32):
                'The precipitation data have the name "Chuva" or "Clima", in the row data from month'
                if k < 10:
                    lista_meses.append(f'Chuva0{k}')
                    lista_meses.append(f'Clima0{k}')
                else:
                    lista_meses.append(f'Chuva{k}')
                    lista_meses.append(f'Clima{k}')
            columns = df.columns
            for a in columns:
                if a not in lista_meses:
                    del df[a]
            self.type_data = "only_mondata"
            self.dataframe = df.drop_duplicates(subset=["Data"], keep="last")

        else:
            return 'List is only True or False'

    def format1(self, comma_to_dot=True, grow=True, years=(0,0)):
        """
        We change the file format to
        1) put date that can be junped
        2) organize the file from to smaler to larger based in time
        3) Chenge the ',' to '.'
        4) Remove same values from file
        5) Change the name of time variable to time
        6) padronize a file to work with Ana, Inemet, Merge and etc
        7) have option to select a series of year start and year end. The date that not exist, are created and set NaN

        :param comma_to_dot: If the person forgot converto comma to point, can do it now
        :param grow: if want the data with the lower in top and greater in end
        :return: Return to self.dataframe the data reorganized by form:

         Data      dia1     dia2    dia3    dia4    dia5    dia6    dia7 ...
                 1993     0.1      0.3     0.7     0.8     0.6     0.4     0.1
                 1993     0.2      0.9     0.5     0.2     0.1     1.9     9.1
                 1993     0.7      0.2     0.9     0.8     0.7     0.4     0.1
                 .
                 .
                 .

                we change to:

                time        pr
                1997-07-01  0.1
                1997-07-02  0.3
                ....
                ...
                ...
                2020-01-01  0.3
                2020-01-02  0.0

        """

        import pandas as pd
        from datetime import timedelta, datetime
        if years != (0,0) and self.type != "format1":
            # Organize the file by date, the lower to greater
            df= self.dataframe
            df['Data'] = pd.to_datetime(df['Data'], format="%d/%m/%Y")
            df = df.drop_duplicates(subset='Data', keep="last")

            if comma_to_dot:
                df = df.replace({',': '.'}, regex=True)
                df = df.reset_index()
                df = df.drop(columns='index')
                self.dataframe = df
            if not grow:
                self.dataframe = df.sort_values(by=['Data'], ascending=True)

            data = np.array([])
            pr = np.array([])
            self.only_mondata()

            # remove date that don't exits
            colum = self.dataframe.columns
            n=0
            for linha in range(len(self.dataframe)):#Follow Rows
                for dia in range(1, 32): #Follow Columns
                    try:
                        temp = self.dataframe[colum[0]][linha]
                        data = np.append(data, pd.to_datetime(datetime(temp.year, temp.month, dia)))  # check if the date is real
                        pr = np.append(pr, float(self.dataframe[colum[dia]][linha]))
                    except:
                        pass

            # chech if have date jump
            a = pd.date_range(start=str(years[0]), end=str(years[1]), freq='D')
            a = a[:-1]
            pr_def = np.full(len(a), np.nan)
            for i in range(len(data)):
                ind = np.argwhere(data[i] == a)
                pr_def[ind] = pr[i]

            df = pd.DataFrame(list(zip(a, pr_def)), columns=['time', 'pr'])
            

            # atualize the format and len
            self.type = "format1"
            df = df[(df.iloc[:, 0].dt.year <= years[1])]
            self.dataframe = df[df.iloc[:, 0].dt.year >= years[0]]
            self.len = len(self.dataframe)
            self.enddate = df.iloc[-1,0] 
            self.startdate = df.iloc[0, 0]


        elif self.type != 'format1': # If the data is not the
            df= self.dataframe
            df['Data'] = pd.to_datetime(df['Data'], format="%d/%m/%Y")
            df = df.drop_duplicates(subset='Data', keep="last")

            if comma_to_dot:
                df = df.replace({',': '.'}, regex=True)
                df = df.reset_index()
                df = df.drop(columns='index')
                self.dataframe = df
            if not grow:
                self.dataframe = df.sort_values(by=['Data'], ascending=True)

            data = np.array([])
            pr = np.array([])
            self.only_mondata()

            # remove data that don't exits
            colum = self.dataframe.columns
            n=0
            for linha in range(len(self.dataframe)):
                for dia in range(1, 32):
                    try:
                        temp = self.dataframe[colum[0]][linha]
                        data = np.append(data, pd.to_datetime(datetime(temp.year, temp.month, dia)))  # check if the date is real
                        pr = np.append(pr, float(self.dataframe[colum[dia]][linha]))
                    except:
                        n+= 0
            a = pd.date_range(start=str(self.enddate.year), end=str(self.enddate.year), freq='D')
            a = a[:-1]
            pr_def = np.full(len(a), np.nan)
            for i in range(len(data)):
                ind = np.argwhere(data[i] == a)
                pr[ind] = pr[i]


            self.dataframe = pd.DataFrame(list(zip(a, pr_def)), columns=['time', 'pr'])

            # atualize the format and len
            self.type = "format1"
            self.len = len(self.dataframe)
            self.enddate = df.iloc[-1,0] 
            self.startdate = df.iloc[0, 0]

    def date(self, line):
        """
        This function get a especific date in a line
        :param linha:
        :return:
        """

        from datetime import datetime
        if self.type_data in ["original", "only_mondata"]:
            return self.dataframe["Data"][line]
        else:
            return self.dataframe['time'][line]
    def where(self, date):
        """
        Return the index where is the date
        :param date: date in datetime.datetime format
        :return:
        """
        self.format1()
        return np.where(self.dataframe()['time'] == date)
    def empty_data(self, type='absolute'):
        if self.type == "format1":
            t = np.sum(np.isnan(self.dataframe.iloc[:, 1]))
            if (type=="absolute"):
                return t
            elif (type=="relative"):
                return (t/self.len)*100
        else:
            self.only_mondata()
            s = 0
            for i,c in enumerate(self.dataframe.columns):
                if c!="Data" and c!= "Chuva31" and c!= "Chuva30" and c!= "Chuva29":
                    s += np.sum(np.isnan(self.dataframe[f'{c}'].to_numpy("float32")))
            print("You are using the original data and this is a estimative, use format1 before to be precise")
            if type == "relative":
                return s
            return (s/(self.enddate-self.startdate).days)*100

    def report(self, path):
            """
            Save a txt file with importants informations

            """
            self.format1()
            with open(path, 'w') as f:
                f.write(f"Station {self.city} {self.code}\n")
                f.write(f"Empty data of : {self.empty_data()}\n")
                f.write(f"Max value: {np.max(self.dataframe.iloc[:, 1])}\n")
                v= self.dataframe.iloc[np.argmax(self.dataframe.iloc[:, 1]),0]
                f.write(f"Date of max value: {v.day}/{v.month}/{v.year}\n")
                f.write(f"Min value: {np.min(self.dataframe.iloc[:, 1])}\n")
                v= self.dataframe.iloc[np.argmin(self.dataframe.iloc[:, 1]),0]
                f.write(f"Date of min value: {v.day}/{v.month}/{v.year}\n")
                f.close()
    def report_mf(self, path_in, path_out, years=(0,0)):
        """
        Save a txt file with imports informations of multiples files

        :param path_in: folder when are the files, only the files
        :param path_out: Folder when the output will be save
        """
    
        files = os.listdir(path_in)
        df = pd.DataFrame(columns=["Code of station", "Period", "Missing data","Missing Data (%)", "Max value", "Date of max"])
        df["Code of station"] = np.arange(len(files))
        for i in range(len(files)):
            ana = Ana(path_in+"/"+files[i])
            if years==(0,0):
                ana.format1()
            else:
                ana.format1(years=years)
            df.iloc[i, 0] = ana.code #Salva o código da estação 
            df.iloc[i, 1] = f"{ana.startdate}-{ana.enddate}" #Salva o período da estação 
            df.iloc[i, 2] = ana.empty_data() #Salva os dados faltantes 
            df.iloc[i, 3] = ana.empty_data(type="relative")#Salva os dados faltantes relativos 
            df.iloc[i, 4] = np.max(ana.dataframe.iloc[:, 1]) #Salva o valor máximo 
            df.iloc[i, 5] = ana.dataframe.iloc[np.argmax(ana.dataframe.iloc[:,1]), 0] # Salva a data do valor máximo
        df.to_csv(path_out)
    # select data or return a data
    def get_year(self,year, with_x=False, return_x0=False):
        """
        From array muti-year, return a year specific
        :param year: can by a Int, or list of Int
        :param with_x:
        :param return_x0: Return the index init
        :return:
        """
        self.format1()
        if isinstance(year, int): # if is only a year
            try:
                a = pd.to_datetime(
                    self.dataframe['time']).dt.year.to_numpy() == year  # array with true or false if the year is that we want
            except:
                raise f"The Data don't have the year {year}"

            a= np.where(a== True)[0] # Return array with index of True and correct the list format
            if with_x:
                return self.dataframe['time'][a], self.dataframe['pr'][a]
            elif return_x0:
                return a[0], self.dataframe['time'][a]
            return self.dataframe['pr'][a]
        else: # is many years
            xf = pd.Series([])
            yf = np.array([])
            for i in year:
                try:
                    a = pd.to_datetime(
                        self.dataframe[
                            'time']).dt.year.to_numpy() == i # array with true or false if the year is that we want
                except:
                    raise f"The Data don't have the year {i}"

                a = np.where(a == True)[0]  # Return array with index of True and correct the list format
                xf = pd.concat([xf, self.dataframe['time'][a]])
                yf = np.concatenate([yf, self.dataframe['pr'][a].to_numpy()])
            if with_x:
                return xf, yf
            elif return_x0:
                raise "This function exist only to one year"
            return yf
    def get_month(self, month, year, with_x=False):
        """
        From array muti-year, return a month specific
        :param month: The month to select as Int, or list of month to get
        :param year: The year to select
        :with_x: return the data, True or Not
        :return: Array with data of month select
        """
        if self.type_data != "format'":
            self.format1()
        #x0 is the index wen start the year that we want
        x0, time = self.get_year(year, return_x0=True)
        if isinstance(month, int):
            try:
                a = time.dt.month.to_numpy() == month  # array with true or false if the year is that we want
            except:
                raise f"The data has't the month {month}"
            a = np.where(a == True)[0]  # Return array with index of True and correct the list format
            a += x0
            if with_x:
                return self.dataframe['time'][a], self.dataframe['pr'][a]
            return self.dataframe['pr'][a]
        else:
            xf = pd.Series([])
            yf = np.array([])
            for i in month:
                try:
                    a = time.dt.month.to_numpy() == i  # array with true or false if the year is that we want
                except:
                    raise f"The data has't the month {i}"
                a = np.where(a == True)[0]  # Return array with index of True and correct the list format
                a += x0
                xf = pd.concat([xf, self.dataframe['time'][a]])
                yf = np.concatenate([yf, self.dataframe['pr'][a].to_numpy()])
            if with_x:
                return xf, yf
            return yf
    def get_lonlat(self, by="Código"):
        """
        The data was get by web http://telemetriaws1.ana.gov.br/EstacoesTelemetricas.aspx in oct 2022
        and is save with name "estacoes.csv"
        and save in self.lat and self.lon, that before are None
        :param by: what will be used to find latitude and longitude
        :return:
        """
        if by=="Código":
            data = pd.read_csv("ana/estacoes.csv")
            data = data.query(f"Código=={self.code}")
            self.lat = float(data["Y"])
            self.lon = float(data["X"])
            self.city = data["Município"]
            self.state = data["Estado"]
            self.status = data["Operando"]

    #Save
    def to_netcdf4(self, path):
        import xarray as xr
        self.format1()

        # pr to float
        df = self.dataframe
        df["pr"] = np.array(df['pr']).astype(float)

        # Run to get lat and lon, create the array of position
        self.get_lonlat()
        lat = [self.lat] * self.len
        lon = [self.lon] * self.len
        df['lat'] = lat
        df['lon'] = lon

        # opening the df with xarray
        ds = df.set_index(['time', 'lat', 'lon']).to_xarray()

        ds = xr.Dataset(ds)
        # add variable attribute metadata
        ds.attrs['lat'] = 'Units: °'
        ds.attrs['lon'] = 'Units: °'
        ds.attrs['time'] = 'daily data of time'

        ds.to_netcdf(path)

    def to_csv(self, path, sep=';', index=False, var="all"):
        if var=="all":
            self.get_info()
            with open(f"{path}", "w") as f:
                f.write(f"var,value\n")
                f.write(f"lon,{self.lon}\n")
                f.write(f"lat, {self.lat}\n")
                f.write(f"alt,{self.alt}\n")
                f.write(f"name,{self.name}\n")
                f.write(f"numcity, {self.NumCity}\n")
                f.write(f"{self.dataframe.to_csv(index=False)}")
        else:
            a = self.dataframe[f"{var}"].to_csv(path, sep=sep, index=index)

    def haversine(self, lon1, lat1):
        """
        Calculate the great circle distance in kilometers between a point and the stations self
        on the earth (specified in decimal degrees)
        
        :param lon1: float, position longitude
        :param lat1: float, position latitude
        """

        lon2, lat2 = self.lon, self.lat 
        from math import cos, sin, asin, sqrt, radians
        # convert decimal degrees to radians
        lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])

        # haversine formula
        dlon = lon2 - lon1
        dlat = lat2 - lat1
        a3 = sin(dlat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(dlon / 2) ** 2
        c = 2 * asin(sqrt(a3))
        r = 6371  # Radius of earth in kilometers. Use 3956 for miles. Determines return value units.
        return c * r

    def plot(self, title='', xlabel='', ylabel=''):
        """
        This function help with fast plot, to more detalhe use the matplotlib
        :param title:
        :param xlabel:
        :param ylabel:
        :return: Return the graphic of data saved in self.dataframe
        """

        import matplotlib.pyplot as plt
        if self.list == False:
            from matplotlib import pyplot as plt
            if self.type != "format1":
                self.format1()

            y = self.dataframe.iloc[:,1]
            x = self.dataframe.iloc[:,0]
            plt.title(title)
            plt.xlabel(xlabel)
            plt.ylabel(ylabel)
            plt.tight_layout()
            plt.plot(x, y)
            plt.show()

        else:
            df = self.positions() # will return a dataframe with code, latitude, longitude
            gdf = gpd.GeoDataFrame(
        df, geometry=gpd.points_from_xy(df.lon, df.lat))
            
            world = gpd.read_file(gpd.datasets.get_path('naturalearth_lowres'))
            ax = world[world.continent == 'South America'].plot(
    color='white', edgecolor='black')

            # We can now plot our ``GeoDataFrame``.
            gdf.plot(ax=ax, color='red')

            plt.show()


    def mean(self, with_x=False, retornar=True):
        """
        Calculate the media when the 'arquivo' is a list of many Ana file
        :param with_x:
        :param retornar:
        :return:
        """
        import os
        from AgenciBr.Ana import Ana
        li = os.listdir(self.path)
        start=datetime.date(2090,12,2)
        end=datetime.date(1902,12,2)
        size=0
        for i in li: # get the size of array
            a = Ana(self.path+'/'+i)
            if a.startdate< start:
                start = a.startdate
            if a.enddate > end:
                end = a.enddate

        x = np.arange(start, end, np.timedelta64(1, "D")) # the time array
        y = np.zeros(x.size)
        d = np.zeros(x.size)

        for i in li:  # get the size of array
            a = Ana(self.path + '/' + i)
            a.format1()
            k2= 0
            for i2, k in enumerate(x):
                if pd.to_datetime(k) == a.dataset['time'][k2]:  # se encontramos o intervalo que é idêntico
                    y[i2] += a.dataset['pr'][k2]
                    d[i2] += 1
                    k2 +=1

        # divide by d
        for k in range(size):

            y[k] = y[k]/d[k]
        if retornar:
            if with_x:
                return x, y
            return y
        self.dataframe = pd.DataFrame(zip(x.tolist(), y.tolist()), columns=['time', 'pr'])
        self.type_data = 'format1'

    # Download data
    def download(self,codigo="NaN", format='csv', dir='', tipo_especifico=False, save_zip=False,
                 state="NaN", city="Nan"):
        """
        Download the Ana data. Code from https://github.com/joaohuf/Ferramentas_HidroWeb
        Data os stations code etc, from http://telemetriaws1.ana.gov.br/EstacoesTelemetricas.aspx
        :param codigo: Code that Ana use to define each station
        :param format: The format of save .mdb, .txt, .csv
        :param dir: The directory to save the file
        :param tipo_especifico:
        :param save_zip: Save file in zip or not
        :param state: Is the acronym of states, example: SP, RJ, SC
        :return:
        """
        import requests
        from io import BytesIO
        from zipfile import ZipFile, BadZipFile

        # Link onde estão os dados das estações convencionaiss convencionais
        BASE_URL = 'http://www.snirh.gov.br/hidroweb/rest/api/documento/convencionais'
        if codigo != "NaN":
            def unzip_station_data(station_raw_data, dir, tipo_especifico):
                try:
                    main_zip_bytes = BytesIO(station_raw_data)
                    main_zip = ZipFile(main_zip_bytes)

                    for inner_file_name in main_zip.namelist():
                        inner_file_content = main_zip.read(inner_file_name)
                        inner_file_bytes = BytesIO(inner_file_content)
                        if not tipo_especifico:
                            with ZipFile(inner_file_bytes, 'r') as zipObject:
                                zipObject.extractall(dir)
                        elif tipo_especifico in inner_file_name:
                            with ZipFile(inner_file_bytes, 'r') as zipObject:
                                zipObject.extractall(dir)
                except BadZipFile:
                    print('No .zip founded')

            # Change the format to Ana format
            if format == 'csv':
                params = {'tipo': 3, 'documentos': codigo}
                r = requests.get(BASE_URL, params=params)
            elif format == 'txt':
                params = {'tipo': 2, 'documentos': codigo}
                r = requests.get(BASE_URL, params=params)
            elif format == 'mdb':
                params = {'tipo': 1, 'documentos': codigo}
                r = requests.get(BASE_URL, params=params)
            else:
                raise "the data format is not csv , mdb or txt"

            # Check if the code exist in Ana file
            if not isinstance(r.content, bytes):
                print(f'Arquivo {codigo} inválído')
                return

            # if save_zip is true, save zip
            if save_zip:
                print(f'Salvando dados {codigo} como zip')
                with open(f'{dir}estacao_{codigo}.zip', 'wb') as f:
                    f.write(r.content)
            # else, unpack each zip file
            else:
                print(f'Extraindo dados da estação {codigo} do zip')
                unzip_station_data(r.content, dir, tipo_especifico)
        else: # we download a list of data
            if state != "NaN":
                estado = f"-{state.upper()}"
                df = pd.read_html("ana/EstacoesTelemetricas_Todas_19_10_2022.xls")
                df = df[0]
                df = df[df["Municipio-UF"].str.contains(estado, na=False)]
                codes = df["CodEstacao"].to_numpy()
                # for each code in list, we download
                for i in codes:
                    a = Ana(self.path, list=True)
                    a.download(codigo=i, format=format, dir=dir,tipo_especifico=False, save_zip=False)
            elif city != "NaN":
                cidade = f"{city.upper()}"
                df = pd.read_html("ana/EstacoesTelemetricas_Todas_19_10_2022.xls")
                df = df[0]
                if state != "NaN":
                    df = df[df["Municipio-UF"].str.contains(f"-{state}", na=False)]
                df = df[df["Municipio-UF"].str.contains(cidade, na=False)]
                codes = df["CodEstacao"].to_numpy()
                # for each code in list, we download
                for i in codes:
                    a = Ana(self.path, list=True)
                    a.download(codigo=i, format=format, dir=dir, tipo_especifico=False, save_zip=False)

    # Extreme index
    def rx5day(self, with_x=False, retornar='number_of_5day_heavy_precipitation_periods_per_time_period'):
        from AgenciBr.indice import Indice
        if self.list:
            fazer=0
        if self.type_data != 'format1':
            self.format1()

        ind = Indice(self.dataframe)
        ind = ind.rx5day(with_x=with_x, retornar=retornar)
        return ind
    def cdd(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.cdd(with_x=with_x)
        return ind
    def cwd(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.cwd(with_x=with_x)
        return ind
    def prcptot(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.prcptot(with_x=with_x)
        return ind
    def prcptot_monthly(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.prcptot_monthly(with_x=with_x)
        return ind
    def r99p(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.r99p(with_x=with_x)
        return ind
    def rnnmm(self, number, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.rnnmm(number=number, with_x=with_x)
        return ind
    def r10mm(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.r10mm(with_x=with_x)
        return ind
    def r20mm(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.r20mm(with_x=with_x)
        return ind
    def r99ptot(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.r99pTOT(with_x=with_x)
        return ind
    def rx1day_anual(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.rx1day_anual(with_x=with_x)
        return ind
    def rx1day_monthly(self, with_x=False):
        from AgenciBr.indice import Indice
        self.format1()
        ind = Indice(self.dataframe)
        ind = ind.rx1day_monthly(with_x=with_x)
        return ind

    #list functions
    def to_climatol(self, path, years):
        """
        homogen('pr', 2003, 2021, expl = TRUE)
        :param path: path to file to save
        :param years: tuple with start year and end year
        :return:
        """
        if self.list == False:
            raise "This function need a list of stations, change list=True and see the documentation"
        else:
            import os
            posi = []
            for i, k in enumerate(os.listdir(self.path)):
                if i == 0:
                    a = Ana(self.path + "/" + k)
                    a.format1(years=years)
                    a.get_lonlat()
                    data = np.array([a.dataframe["pr"]])
                    posi.append([a.lat, a.lon, a.alt, f"{a.code}", f"Ana-{a.code}"])
                else:
                    a = Ana(self.path+"/"+k)
                    a.format1(years=years)
                    a.get_lonlat()
                    # This condiction is to correct a error
                    if len(a.dataframe["pr"]) > len(data[0]) and np.isnan(a.dataframe["pr"][a.len-1]):
                        data = np.append(data, [a.dataframe["pr"][:-1]], axis=0)
                        posi.append([a.lat, a.lon, a.alt, f"{a.code}", f"Ana-{a.code}"])
                    else:
                        data = np.append(data, [a.dataframe["pr"]], axis=0)
                        posi.append([a.lat, a.lon, a.alt, f"{a.code}", f"Ana-{a.code}"])


            a = pd.DataFrame(np.transpose(data))
            b = pd.DataFrame(posi)

            #Before to save, climatol find empty place with some word in string "nan". Because this, we change to str
            a = a.astype(str)
            b = b.astype(str)
            a.to_csv(path+f"/Ttest_{years[0]}-{years[1]}.dat", index=False, header=False, sep=" ")
            a.to_csv(path + f"/pr_{years[0]}-{years[1]}.dat", index=False, header=False, sep=" ")
            b.to_csv(path+f"/pr_{years[0]}-{years[1]}.est", index=False, header=False, sep=" ")

        with open("ana/climatol.r", "w") as climatol:
            climatol.write(f"""
            library(climatol)

            #------------------------------------------------------
            setwd("{self.path}")
            
            dat <- as.matrix(read.table("pr_{years[0]}-{years[1]}.dat"))
            write(dat, "Ttest_{years[0]}-{years[1]}.dat")
            
            homogen('pr', {years[0]}, {years[1]}, expl = TRUE)

            """)
            climatol.close()
        os.system("Rscript ana/climatol.r")

def dist(x1,x2):
    return x1.heaversine(x2.lon, x2.lat)




import os
import xarray as xr
import numpy as np
import pandas as pd

class Alexandre:
    def __init__(self, path, var=None, lon=None, lat=None):
        def opening(path, var):
            if path[-3:] == '.nc': # se é um arquivo individual
                return xr.open_dataset(self.path)
            elif var != None: # A pessoa colocou uma variável para multi arquivos
                return xr.open_mfdataset(self.path + '/'+var + '*.nc')
        self.path = path
        self.var = var
        self.dataset = opening(self.path, var=var)
        self.lat = lat
        self.lon = lon

    def dowload_data(self):
        # ainda construir
        import requests
        def download_file(url, endereco, nome):
            # acessa o site
            resposta = requests.get(url)
            # Acessa o conteúdo e salva em um arquivo
            if resposta.status_code == requests.codes.OK:
                with open(endereco + f'/{nome}.grip2', 'wb') as novo:
                    novo.write(resposta.content)
            else:
                resposta.raise_for_status()

    def open_mfdataset(self, var):
        """
        Open many files .nc in one.
        :param var: string with name of variable
        :return:
        """
        self.dataset = xr.open_mfdataset(self.path + var + '*.nc')

    def open_dataset(self):
        """
        Open the dataset
        :return:
        """
        self.dataset = xr.open_dataset(self.path)

    def sum_month(self, v):
        # testar
        import datetime
        df = self.dataset
        mes = pd.to_datetime(df.time.values[0]).month  # primeiro mês
        soma = 0
        t = var = np.array([])
        size = len(df.time.values)
        ano = pd.to_datetime(df.time.values[0]).year
        for k in range(len(df.time.values)):
            if pd.to_datetime(df.time.values[k]).month != mes:
                if mes == 12:
                    var = np.append(var, soma)
                    soma = 0
                    mes = 1
                    ano += 1
                    t = np.append(t, datetime.datetime(ano, mes, 1))
                    print(f'Se foram {round(100 * k / size, 2)} %')
                else:
                    var = np.append(var, soma)
                    soma = 0
                    mes += 1
                    t = np.append(t, datetime.datetime(ano, mes, 1))
            soma += df[v].values.transpose()[0][k]
        return xr.DataArray(var,coords=[t], dims=["time"])

    def to_csv(self, name, var=None):
        """
        Convert the file in .csv files
        :param name:
        :param var:
        :return:
        """
        if var == None:
            raise "Digite a variável (var) que será usada quando abre o arquivo"
        if isinstance(var, list): # Se é uma lista de variáveis
            if self.lon and self.lat != None:
                for n, var_name2get in enumerate(var):
                    var2get_xr = xr.open_mfdataset(self.path + var_name2get + '*.nc')
                    # a = var2get_xr.to_dataframe()
                    if n == 0:
                        var_ar = var2get_xr[var_name2get].sel(longitude=self.lon,latitude=self.lat,method='nearest').values

                        var_ar = [var_ar.tolist()]
                        time = [var2get_xr.time.values]
                    else:

                        t = var2get_xr[var_name2get].sel(longitude=self.lon,latitude=self.lat,method='nearest').values
                        var_ar = var_ar.append(t.tolist())
                        del t
                for n in range(1):
                    file = var_ar
                    pd.DataFrame(file, index=time, columns=var).to_csv(name, float_format='%.1f')

        if self.lon== None and self.lat == None:
            c=0
            for lat in range(self.dataset.lat.values):
                for lon in range(self.dataset.lon.values):
                    if c==0: # Caso inicial
                        a = self.dataset[var].sel(longitude=xr.DataArray(lon, dims='z'),
                                                  latitude=xr.DataArray(lat, dims='z'),
                                                  method='nearest').values
                        final = np.array(a)
                        del a
                    else: # Casos seguintes
                        a = self.dataset[var].sel(longitude=xr.DataArray(lon, dims='z'),
                                      latitude=xr.DataArray(lat, dims='z'),
                              method='nearest').values
                        final = np.c_[final, a]
                        del a
            if ~np.isnan(final[:, 0]):
                file = final[:, 0]
                pd.DataFrame(file, index=time, columns=var).to_csv(name, float_format='%.1f')

        else:
            a = self.dataset[var].sel(longitude=xr.DataArray([self.lon], dims='z'),
                              latitude=xr.DataArray([self.lat], dims='z'),
                              method='nearest').values
            final= np.array(a)
            del a
            pd.DataFrame(list(zip(self.dataset.time.values,final[:,0])), columns=['time',var]).to_csv(name, index=False)
    def to_netcdf(self, path):
        if self.var == None:
            raise "Type a variable (var) that will be used when open the file"
        t = self.dataset[self.var].sel(longitude=xr.DataArray([self.lon], dims='z'),
                              latitude=xr.DataArray([self.lat], dims='z'),
                              method='nearest').to_netcdf(path)

    #Climate index
    def prcptot(self, with_x=False):
        import Indice
        if self.var != 'pr':
            raise "Is necessary to use the precipitation data (pr)"
        self.to_csv('temp.nc', var=self.var)
        a = indice.indice(pd.read_csv('temp.nc'))
        x,y = a.prcptot(with_x=with_x)
        os.remove('temp.nc')
        return x,y
    def cwd(self, with_x=False):
        import Indice
        if self.var != 'pr':
            raise "Is necessary to use the precipitation data (pr)"
        self.to_csv('temp.nc', var=self.var)
        a = indice.indice(pd.read_csv('temp.nc'))
        x, y = a.cwd(with_x=with_x)
        os.remove('temp.nc')
        return x, y
    def cdd(self, with_x=False):
        import Indice
        if self.var != 'pr':
            raise "Is necessary to use the precipitation data (pr)"
        self.to_csv('temp.nc', var=self.var)
        a = indice.indice(pd.read_csv('temp.nc'))
        x, y = a.cdd(with_x=with_x)
        os.remove('temp.nc')
        return x, y
    def rx5day(self, with_x=False, time='y'):
        import Indice
        if self.var != 'pr':
            raise "Is necessary to use the precipitation data (pr)"
        self.to_csv('temp.nc', var=self.var)
        a = indice.indice(pd.read_csv('temp.nc'))
        x, y = a.rx5day(with_x=with_x)
        os.remove('temp.nc')
        return x, y
    def tx(self, with_x=False):
        import Indice
        print(self.var)
        if self.var.lower in ['t_maxmin' ,'tmax']:
            raise "Is necessary to use the maximum temperature data (Tmax)"
        self.to_csv('temp.nc', var=self.var)
        a = indice.indice(pd.read_csv('temp.nc'))
        x, y = a.TX(with_x=with_x, var= self.var)
        os.remove('temp.nc')
        return x, y
    def tn(self, with_x=False):
        import Indice
        if self.var.lower in ['t_maxmin', 'tmin']:
            raise "Is necessary to use the minimum temperature data (Tmin)"
        self.to_csv('temp.nc', var=self.var)
        a = indice.indice(pd.read_csv('temp.nc'))

        x, y = a.TN(with_x=with_x, var=self.var)
        os.remove('temp.nc')
        return x, y
    def tx90p(self, with_x=False):
        import Indice
        if self.var.lower in ['t_maxmin', 'tmax']:
            raise "Is necessary to use the maximum temperature data (Tmax)"
        self.to_csv('temp.nc', var=self.var)
        a = indice.indice(pd.read_csv('temp.nc'))
        x, y = a.TX90p(with_x=with_x, var=self.var)
        os.remove('temp.nc')
        return x, y
    def tn90p(self, with_x=False):
        import Indice
        if self.var.lower in ['t_maxmin', 'tmax']:
            raise "Is necessary to use the maximum temperature data (Tmax)"
        self.to_csv('temp.nc', var=self.var)
        a = indice.indice(pd.read_csv('temp.nc'))
        x, y = a.TN90p(with_x=with_x, var=self.var)
        os.remove('temp.nc')
        return x, y




from setuptools import setup

from setuptools import setup

with open("README.md", "r") as arq:
    readme = arq.read()

setup(name='AgenciBr',
    version='0.1.8',
    license='MIT License',
    author='Thiago Santos',
    long_description=readme,
    long_description_content_type="text/markdown",
    author_email='tthiagosantos38@gmail.com',
    keywords='Ana, Inemet, Ideam',
    description=u'Package to work with data from brazilian agenci',
    packages=['AgenciBr'],
    install_requires=['matplotlib',"numpy","pandas","requests","datetime","xarray","netCDF4","cfgrib", "geopandas"],)
